@@ -23,9 +23,9 @@ include_once 'Horde/Form/Renderer.php';
  * configuration of Horde applications, writing conf.php files from
  * conf.xml source files, generating user interfaces, etc.
  *
- * $Horde: framework/Horde/Horde/Config.php,v 1.80.2.34 2008/05/27 02:15:48 chuck Exp $
+ * $Horde: framework/Horde/Horde/Config.php,v 1.80.2.40 2009/02/25 05:35:42 chuck Exp $
  *
- * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -527,6 +527,14 @@ class Horde_Config {
                                      'is_default' => $this->_isDefaultRaw($curctx, $this->_getNodeOnlyText($node)));
                 break;
 
+            case 'configsecret':
+                $conf[$name] = array('_type' => 'text',
+                                     'required' => true,
+                                     'desc' => $desc,
+                                     'default' => $this->_default($curctx, sha1(uniqid(mt_rand(), true))),
+                                     'is_default' => $this->_isDefault($curctx, $this->_getNodeOnlyText($node)));
+                break;
+
             case 'configsql':
                 $conf[$node->get_attribute('switchname')] = $this->_configSQL($ctx, $node);
                 break;
@@ -639,8 +647,17 @@ class Horde_Config {
             '_type' => 'text',
             'required' => true,
             'desc' => 'Internally used charset',
-            'default' => $this->_default($ctx . '|charset', 'iso-8859-1'));
-
+            'default' => $this->_default($ctx . '|charset', 'utf-8'));
+        $ssl = array(
+            '_type' => 'boolean',
+            'required' => false,
+            'desc' => 'Use SSL to connect to the server?',
+            'default' => $this->_default($ctx . '|ssl', false));
+        $ca = array(
+            '_type' => 'text',
+            'required' => false,
+            'desc' => 'Certification Authority to use for SSL connections',
+            'default' => $this->_default($ctx . '|ca', ''));
         $oci8_fields = array(
             'persistent' => $persistent,
             'username' => $username,
@@ -683,8 +700,12 @@ class Horde_Config {
                     'desc' => 'Enabled',
                     'fields' => array(
                         'read' => array(
-                            'hostspec' => $read_hostspec,
-                            'port' => $read_port)))));
+                            'persistent' => $persistent,
+                            'username' => $username,
+                            'password' => $password,
+                            'protocol' => $mysql_protocol,
+                            'database' => $database,
+                            'charset' => $charset)))));
 
         $custom_fields = array(
             'required' => true,
@@ -789,6 +810,8 @@ class Horde_Config {
                         'protocol' => $mysql_protocol,
                         'database' => $database,
                         'charset' => $charset,
+                        'ssl' => $ssl,
+                        'ca' => $ca,
                         'splitread' => $splitread)),
                 'mysqli' => array(
                     'desc' => 'MySQL (mysqli)',
@@ -798,7 +821,10 @@ class Horde_Config {
                         'protocol' => $mysql_protocol,
                         'database' => $database,
                         'charset' => $charset,
-                        'splitread' => $splitread)),
+                        'splitread' => $splitread,
+                        'ssl' => $ssl,
+                        'ca' => $ca
+            )),
                 'oci8' => array(
                     'desc' => 'Oracle',
                     'fields' => $oci8_fields),
@@ -1084,6 +1110,7 @@ class Horde_Config {
      * Returns a certain value from the current configuration file or
      * a default value, if not found, and which of the values have been
      * returned.
+     *
      * It does NOT return the actual value, but the PHP expression as used
      * in the configuration file.
      *
@@ -1137,6 +1164,7 @@ class Horde_Config {
     /**
      * Returns an associative array containing all possible values of the
      * specified <configenum> tag.
+     *
      * The keys contain the actual enum values while the values contain their
      * corresponding descriptions.
      *
@@ -1322,14 +1350,14 @@ class ConfigForm extends Horde_Form {
      * Constructor.
      *
      * @param Variables &$vars  The Variables object of this form.
-     * @param string $app             The name of the application that this
-     *                                configuration form is for.
+     * @param string $app       The name of the application that this
+     *                          configuration form is for.
      */
     function ConfigForm(&$vars, $app)
     {
         parent::Horde_Form($vars);
 
-        $this->_xmlConfig = &new Horde_Config($app);
+        $this->_xmlConfig = new Horde_Config($app);
         $this->_vars = &$vars;
         $config = $this->_xmlConfig->readXMLConfig();
         $this->addHidden('', 'app', 'text', true);
