@@ -31,7 +31,7 @@ define('HORDE_DATE_MASK_ALLPARTS', 63);
  * Horde Date wrapper/logic class, including some calculation
  * functions.
  *
- * $Horde: framework/Date/Date.php,v 1.8.10.15 2008/04/24 03:18:24 bklang Exp $
+ * $Horde: framework/Date/Date.php,v 1.8.10.19 2009/12/10 12:19:58 jan Exp $
  *
  * @package Horde_Date
  */
@@ -87,8 +87,17 @@ class Horde_Date {
     var $_supportedSpecs = '%CdDeHImMnRStTyY';
 
     /**
-     * Build a new date object. If $date contains date parts, use them
-     * to initialize the object.
+     * Build a new date object. If $date contains date parts, use them to
+     * initialize the object.
+     *
+     * Recognized formats:
+     * - arrays with keys 'year', 'month', 'mday', 'day' (since Horde 3.2),
+     *   'hour', 'min', 'minute' (since Horde 3.2), 'sec'
+     * - objects with properties 'year', 'month', 'mday', 'hour', 'min', 'sec'
+     * - yyyy-mm-dd hh:mm:ss (since Horde 3.1)
+     * - yyyymmddhhmmss (since Horde 3.1)
+     * - yyyymmddThhmmssZ (since Horde 3.1.4)
+     * - unix timestamps
      */
     function Horde_Date($date = null)
     {
@@ -137,26 +146,6 @@ class Horde_Date {
         }
     }
 
-    /**
-     * Return the unix timestamp representation of this date.
-     *
-     * @return integer  A unix timestamp.
-     */
-    function timestamp()
-    {
-        return Horde_Date::_mktime($this->hour, $this->min, $this->sec, $this->month, $this->mday, $this->year);
-    }
-
-    /**
-     * Return the unix timestamp representation of this date, 12:00am.
-     *
-     * @return integer  A unix timestamp.
-     */
-    function datestamp()
-    {
-        return Horde_Date::_mktime(0, 0, 0, $this->month, $this->mday, $this->year);
-    }
-
     /**
      * @static
      */
@@ -268,25 +257,26 @@ class Horde_Date {
      */
     function weekOfYear()
     {
-        return date('W', $this->timestamp());
+        return $this->format('W');
     }
 
     /**
      * Return the number of weeks in the given year (52 or 53).
      *
+     * @static
+     *
      * @param integer $year  The year to count the number of weeks in.
      *
-     * @return integer $numWeeks      The number of weeks in $year.
+     * @return integer $numWeeks   The number of weeks in $year.
      */
     function weeksInYear($year)
     {
         // Find the last Thursday of the year.
         $day = 31;
-        while (date('w', Horde_Date::_mktime(0, 0, 0, 12, $day, $year)) != HORDE_DATE_THURSDAY) {
-            --$day;
+        $date = new Horde_Date(array('year' => $year, 'month' => 12, 'mday' => $day, 'hour' => 0, 'min' => 0, 'sec' => 0));
+        while ($date->dayOfWeek() != HORDE_DATE_THURSDAY) {
+            --$date->mday;
         }
-
-        $date = new Horde_Date(array('year' => $year, 'month' => 12, 'mday' => $day));
         return $date->weekOfYear();
     }
 
@@ -486,7 +476,7 @@ class Horde_Date {
      */
     function tzOffset($colon = true)
     {
-        $secs = date('Z', $this->timestamp());
+        $secs = $this->format('Z');
 
         if ($secs < 0) {
             $sign = '-';
@@ -500,6 +490,59 @@ class Horde_Date {
                        $sign, $mins / 60, $colon, $mins % 60);
     }
 
+    /**
+     * Return the unix timestamp representation of this date.
+     *
+     * @return integer  A unix timestamp.
+     */
+    function timestamp()
+    {
+        if (class_exists('DateTime')) {
+            return $this->format('U');
+        } else {
+            return Horde_Date::_mktime($this->hour, $this->min, $this->sec, $this->month, $this->mday, $this->year);
+        }
+    }
+
+    /**
+     * Return the unix timestamp representation of this date, 12:00am.
+     *
+     * @return integer  A unix timestamp.
+     */
+    function datestamp()
+    {
+        if (class_exists('DateTime')) {
+            $dt = new DateTime();
+            $dt->setDate($this->year, $this->month, $this->mday);
+            $dt->setTime(0, 0, 0);
+            return $dt->format('U');
+        } else {
+            return Horde_Date::_mktime(0, 0, 0, $this->month, $this->mday, $this->year);
+        }
+    }
+
+    /**
+     * Format time using the specifiers available in date() or in the DateTime
+     * class' format() method.
+     *
+     * @since Horde 3.3
+     *
+     * @param string $format
+     *
+     * @return string  Formatted time.
+     */
+    function format($format)
+    {
+        if (class_exists('DateTime')) {
+            $dt = new DateTime();
+            $dt->setDate($this->year, $this->month, $this->mday);
+            $dt->setTime($this->hour, $this->min, $this->sec);
+            return $dt->format($format);
+        } else {
+            return date($format, $this->timestamp());
+        }
+    }
+
     /**
      * Format time in ISO-8601 format. Works correctly since Horde 3.2.
      *
@@ -517,18 +560,32 @@ class Horde_Date {
      */
     function rfc2822DateTime()
     {
-        return date('D, j M Y H:i:s ', $this->timestamp()) . $this->tzOffset(false);
+        return $this->format('D, j M Y H:i:s') . ' ' . $this->tzOffset(false);
     }
 
     /**
      * Format time in RFC 3339 format.
      *
+     * @since Horde 3.1
+     *
      * @return string  Date and time in RFC 3339 format. The seconds part has
      *                 been added with Horde 3.2.
      */
     function rfc3339DateTime()
     {
-        return date('Y-m-d\TH:i:s', $this->timestamp());
+        return $this->format('Y-m-d\TH:i:s');
+    }
+
+    /**
+     * Format time in the format we use for SQL queries.
+     *
+     * @since Horde 3.3.6
+     *
+     * @return string  Date and time in SQL format.
+     */
+    function sqlDateTime()
+    {
+        return $this->format('Y-m-d H:i:s');
     }
 
     /**
@@ -536,14 +593,16 @@ class Horde_Date {
      *
      * @return string  Date and time.
      */
-    function cTime($time = false)
+    function cTime()
     {
-        return date('D M j H:i:s Y', $this->timestamp());
+        return $this->format('D M j H:i:s Y');
     }
 
     /**
      * Format date and time using strftime() format.
      *
+     * @since Horde 3.1
+     *
      * @return string  strftime() formatted date and time.
      */
     function strftime($format)
@@ -616,9 +675,9 @@ class Horde_Date {
      * mktime() implementation that supports dates outside of 1970-2038,
      * from http://phplens.com/phpeverywhere/adodb_date_library.
      *
-     * This does NOT work with pre-1970 daylight saving times.
+     * @TODO remove in Horde 4
      *
-     * @TODO Remove in Horde 4
+     * This does NOT work with pre-1970 daylight saving times.
      *
      * @static
      */
@@ -631,9 +690,8 @@ class Horde_Date {
                 : @mktime($hr, $min, $sec);
         }
 
-        if ($year > 1901 &&
-            (version_compare(PHP_VERSION, '5.0.0', '>=') ||
-             (1971 < $year && $year < 2038))) {
+        if ($year > 1901 && $year < 2038 &&
+            ($year >= 1970 || version_compare(PHP_VERSION, '5.0.0', '>='))) {
             return $is_gmt
                 ? @gmmktime($hr, $min, $sec, $mon, $day, $year)
                 : @mktime($hr, $min, $sec, $mon, $day, $year);
