@@ -2,9 +2,9 @@
 /**
  * This file contains the Horde_Date_Recurrence class and according constants.
  *
- * $Horde: framework/Date/Date/Recurrence.php,v 1.7.2.6 2008/05/17 20:55:38 jan Exp $
+ * $Horde: framework/Date/Date/Recurrence.php,v 1.7.2.15 2009/12/03 11:17:26 wrobel Exp $
  *
- * Copyright 2007-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2007-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -102,6 +102,13 @@ class Horde_Date_Recurrence {
      */
     var $exceptions = array();
 
+    /**
+     * All the dates this recurrence has been marked as completed.
+     *
+     * @var array
+     */
+    var $completions = array();
+
     /**
      * Constructor.
      *
@@ -308,9 +315,8 @@ class Horde_Date_Recurrence {
      */
     function hasRecurEnd()
     {
-        return isset($this->recurEnd)
-            && isset($this->recurEnd->year)
-            && $this->recurEnd->year != 9999;
+        return isset($this->recurEnd) && isset($this->recurEnd->year) &&
+            $this->recurEnd->year != 9999;
     }
 
     /**
@@ -368,19 +374,67 @@ class Horde_Date_Recurrence {
 
             $diff = Date_Calc::dateDiff($start_week->mday, $start_week->month, $start_week->year,
                                         $after_week->mday, $after_week->month, $after_week->year);
-            $recur = $diff + ($diff % ($this->recurInterval * 7));
-            if ($this->recurCount &&
-                ceil($recur / 7) / $this->recurInterval >= $this->recurCount) {
-                return false;
+            $interval = $this->recurInterval * 7;
+            $repeats = floor($diff / $interval);
+            if ($diff % $interval < 7) {
+                $recur = $diff;
+            } else {
+                /**
+                 * If the after_week is not in the first week interval the
+                 * search needs to skip ahead a complete interval. The way it is
+                 * calculated here means that an event that occurs every second
+                 * week on Monday and Wednesday with the event actually starting
+                 * on Tuesday or Wednesday will only have one incidence in the
+                 * first week.
+                 */
+                $recur = $interval * ($repeats + 1);
             }
+
+            if ($this->hasRecurCount()) {
+                $recurrences = 0;
+                /**
+                 * Correct the number of recurrences by the number of events
+                 * that lay between the start of the start week and the
+                 * recurrence start.
+                 */
+                $next = new Horde_Date($start_week);
+                while ($next->compareDateTime($this->start) < 0) {
+                    if ($this->recurOnDay((int)pow(2, $next->dayOfWeek()))) {
+                        $recurrences--;
+                    }
+                    ++$next->mday;
+                    $next->correct();
+                }
+                if ($repeats > 0) {
+                    $weekdays = $this->recurData;
+                    $total_recurrences_per_week = 0;
+                    while ($weekdays > 0) {
+                        if ($weekdays % 2) {
+                            $total_recurrences_per_week++;
+                        }
+                        $weekdays = ($weekdays - ($weekdays % 2)) / 2;
+                    }
+                    $recurrences += $total_recurrences_per_week * $repeats;
+                }
+            }
+
             $next = $start_week;
             list($next->mday, $next->month, $next->year) = explode('/', Date_Calc::daysToDate(Date_Calc::dateToDays($next->mday, $next->month, $next->year) + $recur, '%e/%m/%Y'));
             $next = new Horde_Date($next);
             while ($next->compareDateTime($after) < 0 &&
                    $next->compareDateTime($after_week_end) < 0) {
+                if ($this->hasRecurCount()
+                    && $next->compareDateTime($after) < 0
+                    && $this->recurOnDay((int)pow(2, $next->dayOfWeek()))) {
+                    $recurrences++;
+                }
                 ++$next->mday;
                 $next->correct();
             }
+            if ($this->hasRecurCount() &&
+                $recurrences >= $this->recurCount) {
+                return false;
+            }
             if (!$this->hasRecurEnd() ||
                 $next->compareDateTime($this->recurEnd) <= 0) {
                 if ($next->compareDateTime($after_week_end) >= 0) {
@@ -640,7 +694,8 @@ class Horde_Date_Recurrence {
 
         $next = $this->nextRecurrence(new Horde_Date($this->start));
         while (is_object($next)) {
-            if (!$this->hasException($next->year, $next->month, $next->mday)) {
+            if (!$this->hasException($next->year, $next->month, $next->mday) &&
+                !$this->hasCompletion($next->year, $next->month, $next->mday)) {
                 return true;
             }
 
@@ -655,6 +710,31 @@ class Horde_Date_Recurrence {
         return false;
     }
 
+    /**
+     * Returns the next active recurrence.
+     *
+     * @param Horde_Date $afterDate  Return events after this date.
+     *
+     * @return Horde_Date|boolean The date of the next active
+     *                             recurrence or false if the event
+     *                             has no active recurrence after
+     *                             $afterDate.
+     */
+    function nextActiveRecurrence($afterDate)
+    {
+        $next = $this->nextRecurrence($afterDate);
+        while (is_object($next)) {
+            if (!$this->hasException($next->year, $next->month, $next->mday) &&
+                !$this->hasCompletion($next->year, $next->month, $next->mday)) {
+                return $next;
+            }
+            $next->mday++;
+            $next = $this->nextRecurrence($next);
+        }
+
+        return false;
+    }
+
     /**
      * Adds an exception to a recurring event.
      *
@@ -708,6 +788,59 @@ class Horde_Date_Recurrence {
         return $this->exceptions;
     }
 
+    /**
+     * Adds a completion to a recurring event.
+     *
+     * @param integer $year   The year of the execption.
+     * @param integer $month  The month of the execption.
+     * @param integer $mday   The day of the month of the completion.
+     */
+    function addCompletion($year, $month, $mday)
+    {
+        $this->completions[] = sprintf('%04d%02d%02d', $year, $month, $mday);
+    }
+
+    /**
+     * Deletes a completion from a recurring event.
+     *
+     * @param integer $year   The year of the execption.
+     * @param integer $month  The month of the execption.
+     * @param integer $mday   The day of the month of the completion.
+     */
+    function deleteCompletion($year, $month, $mday)
+    {
+        $key = array_search(sprintf('%04d%02d%02d', $year, $month, $mday), $this->completions);
+        if ($key !== false) {
+            unset($this->completions[$key]);
+        }
+    }
+
+    /**
+     * Checks if a completion exists for a given reccurence of an event.
+     *
+     * @param integer $year   The year of the reucrance.
+     * @param integer $month  The month of the recurrance.
+     * @param integer $mday   The day of the month of the recurrance.
+     *
+     * @return boolean  True if a completion exists for the given date.
+     */
+    function hasCompletion($year, $month, $mday)
+    {
+        return in_array(sprintf('%04d%02d%02d', $year, $month, $mday),
+                        $this->getCompletions());
+    }
+
+    /**
+     * Retrieves all the completions for this event.
+     *
+     * @return array  Array containing the dates of all the completions in
+     *                YYYYMMDD form.
+     */
+    function getCompletions()
+    {
+        return $this->completions;
+    }
+
     /**
      * Parses a vCalendar 1.0 recurrence rule.
      *
@@ -836,21 +969,12 @@ class Horde_Date_Recurrence {
             break;
 
         case HORDE_DATE_RECUR_MONTHLY_WEEKDAY:
-            $next_week = new Horde_Date($this->start);
-            $next_week->mday += 7;
-            $next_week->correct();
-
-            if ($this->start->month != $next_week->month) {
-                $p = 5;
-            } else {
-                $p = (int)($this->start->mday / 7);
-                if (($this->start->mday % 7) > 0) {
-                    $p++;
-                }
+            $nth_weekday = (int)($this->start->mday / 7);
+            if (($this->start->mday % 7) > 0) {
+                    $nth_weekday++;
             }
-
             $vcaldays = array('SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA');
-            $rrule = 'MP' . $this->recurInterval . ' ' . $p . '+ ' . $vcaldays[$this->start->dayOfWeek()];
+            $rrule = 'MP' . $this->recurInterval . ' ' . $nth_weekday . '+ ' . $vcaldays[$this->start->dayOfWeek()];
             break;
 
         case HORDE_DATE_RECUR_YEARLY_DATE:
@@ -865,14 +989,19 @@ class Horde_Date_Recurrence {
             return '';
         }
 
-        return $this->hasRecurEnd() ?
-            $rrule . ' ' . $calendar->_exportDate($this->recurEnd) :
-            $rrule . ' #' . (int)$this->getRecurCount();
+        if ($this->hasRecurEnd()) {
+            $recurEnd = new Horde_Date($this->recurEnd);
+            $recurEnd->mday++;
+            return $rrule . ' ' . $calendar->_exportDateTime($recurEnd);
+        }
+
+        return $rrule . ' #' . (int)$this->getRecurCount();
     }
 
     /**
      * Parses an iCalendar 2.0 recurrence rule.
      *
+     * @link http://rfc.net/rfc2445.html#s4.3.10
      * @link http://rfc.net/rfc2445.html#s4.8.5
      * @link http://www.shuchow.com/vCalAddendum.html
      *
@@ -966,6 +1095,7 @@ class Horde_Date_Recurrence {
     /**
      * Creates an iCalendar 2.0 recurrence rule.
      *
+     * @link http://rfc.net/rfc2445.html#s4.3.10
      * @link http://rfc.net/rfc2445.html#s4.8.5
      * @link http://www.shuchow.com/vCalAddendum.html
      *
@@ -1003,20 +1133,13 @@ class Horde_Date_Recurrence {
             break;
 
         case HORDE_DATE_RECUR_MONTHLY_WEEKDAY:
-            $next_week = new Horde_Date($this->start);
-            $next_week->mday += 7;
-            $next_week->correct();
-            if ($this->start->month != $next_week->month) {
-                $p = 5;
-            } else {
-                $p = (int)($this->start->mday / 7);
-                if (($this->start->mday % 7) > 0) {
-                    $p++;
-                }
+            $nth_weekday = (int)($this->start->mday / 7);
+            if (($this->start->mday % 7) > 0) {
+                $nth_weekday++;
             }
             $vcaldays = array('SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA');
             $rrule = 'FREQ=MONTHLY;INTERVAL=' . $this->recurInterval
-                . ';BYDAY=' . $p . $vcaldays[$this->start->dayOfWeek()];
+                . ';BYDAY=' . $nth_weekday . $vcaldays[$this->start->dayOfWeek()];
             break;
 
         case HORDE_DATE_RECUR_YEARLY_DATE:
@@ -1029,22 +1152,23 @@ class Horde_Date_Recurrence {
             break;
 
         case HORDE_DATE_RECUR_YEARLY_WEEKDAY:
+            $nth_weekday = (int)($this->start->mday / 7);
+            if (($this->start->mday % 7) > 0) {
+                $nth_weekday++;
+            }
             $vcaldays = array('SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA');
-            $weekday =  date('W', mktime(0,
-                                         0,
-                                         0,
-                                         $this->start->month,
-                                         1,
-                                         $this->start->year));
             $rrule = 'FREQ=YEARLY;INTERVAL=' . $this->recurInterval
-                . ';BYDAY=' . ($this->start->weekOfYear() - $weekday + 1)
+                . ';BYDAY='
+                . $nth_weekday
                 . $vcaldays[$this->start->dayOfWeek()]
                 . ';BYMONTH=' . $this->start->month;
             break;
         }
 
         if ($this->hasRecurEnd()) {
-            $rrule .= ';UNTIL=' . $calendar->_exportDate($this->recurEnd);
+            $recurEnd = new Horde_Date($this->recurEnd);
+            $recurEnd->mday++;
+            $rrule .= ';UNTIL=' . $calendar->_exportDateTime($recurEnd);
         }
         if ($count = $this->getRecurCount()) {
             $rrule .= ';COUNT=' . $count;
@@ -1164,7 +1288,7 @@ class Horde_Date_Recurrence {
             break;
 
         case 'date':
-            $recur_end = new Horde_Date(Kolab_Date::decodeDate($hash['range']));
+            $recur_end = new Horde_Date($hash['range']);
             $recur_end->hour = 23;
             $recur_end->min = 59;
             $recur_end->sec = 59;
@@ -1253,12 +1377,13 @@ class Horde_Date_Recurrence {
             $this->start->correct();
         }
 
-        // Exclusions.
-        if (isset($hash['exclusion'])) {
-            foreach($hash['exclusion'] as $exclusion) {
-                $date = new Horde_Date(Kolab_Date::decodeDate($exclusion));
-                $this->addException($date->year, $date->month, $date->mday);
-            }
+        // Exceptions.
+        if (isset($hash['exceptions'])) {
+            $this->exceptions = $hash['exceptions'];
+        }
+
+        if (isset($hash['completions'])) {
+            $this->completions = $hash['completions'];
         }
 
         return true;
@@ -1271,6 +1396,10 @@ class Horde_Date_Recurrence {
      */
     function toHash()
     {
+        if ($this->getRecurType() == HORDE_DATE_RECUR_NONE) {
+            return array();
+        }
+
         $day2number = array(
             0 => 'sunday',
             1 => 'monday',
@@ -1363,21 +1492,15 @@ class Horde_Date_Recurrence {
         } elseif ($this->hasRecurEnd()) {
             $date = $this->getRecurEnd();
             $hash['range-type'] = 'date';
-            $hash['range'] = Kolab_Date::encodeDate($date->datestamp());
+            $hash['range'] = $date->datestamp();
         } else {
             $hash['range-type'] = 'none';
             $hash['range'] = '';
         }
 
-        // Recurrence exclusions.
-        $exclusions = array();
-        foreach ($this->getExceptions() as $exception) {
-            if (!empty($exception)) {
-                list($year, $month, $mday) = sscanf($exception, '%04d%02d%02d');
-                $exclusions[] = "$year-$month-$mday";
-            }
-        }
-        $hash['exclusion'] = $exclusions;
+        // Recurrence exceptions
+        $hash['exceptions'] = $this->exceptions;
+        $hash['completions'] = $this->completions;
 
         return $hash;
     }
