@@ -4,10 +4,10 @@
 require_once 'Horde.php';
 
 /** We need access to the Kolab IMAP storage */
-require_once 'Horde/Kolab/IMAP.php';
+require_once 'Horde/Kolab/Deprecated.php';
 
 /** We need the Kolab date functions */
-require_once 'Horde/Kolab/Date.php';
+require_once 'Horde/Kolab/Format/Date.php';
 
 /**
  * The Horde_Kolab library is both an object used by application drivers to
@@ -15,9 +15,9 @@ require_once 'Horde/Kolab/Date.php';
  * several functions to help in the IMAP folder <-> Horde Share synchronisation
  * process.
  *
- * $Horde: framework/Kolab/Kolab.php,v 1.26.10.27 2008/06/05 07:08:13 wrobel Exp $
+ * $Horde: framework/Kolab/Kolab.php,v 1.26.10.32 2009/01/06 15:23:13 jan Exp $
  *
- * Copyright 2004-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2004-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -30,6 +30,8 @@ class Kolab {
     /**
      * The current application that this Kolab object instance is catering to.
      *
+     * @deprecated
+     *
      * @var string
      */
     var $_app;
@@ -37,6 +39,8 @@ class Kolab {
     /**
      * The storage driver for the Kolab server.
      *
+     * @deprecated
+     *
      * @var Kolab_Storage
      */
     var $_storage;
@@ -44,6 +48,8 @@ class Kolab {
     /**
      * Indicates the version of this driver
      *
+     * @deprecated
+     *
      * @var int
      */
     var $version = 2;
@@ -68,6 +74,11 @@ class Kolab {
      */
     var $_uid;
 
+    /**
+     *
+     * @deprecated
+     *
+     */
     function Kolab($app = null)
     {
         if (!isset($app)) {
@@ -76,12 +87,14 @@ class Kolab {
         }
         $this->_app = $app;
         
-        $this->_storage = &new Kolab_IMAP();
+        $this->_storage = &new Kolab_Storage_Deprecated();
     }
 
     /**
      * Return the uid of the message we are currently dealing with.
      *
+     * @deprecated
+     *
      * @return string  The Kolab UID of the message we are currently
      *                 dealing with.
      */
@@ -93,6 +106,8 @@ class Kolab {
     /**
      * Open the specified share.
      *
+     * @deprecated
+     *
      * @param string $share_uid      The uid of the share that
      *                               should be opened.
      * @param int    $loader         The version of the XML
@@ -112,6 +127,8 @@ class Kolab {
 
     /**
      * Close the current share.
+     *
+     * @deprecated
      */
     function close()
     {
@@ -121,6 +138,8 @@ class Kolab {
     /**
      * Retrieve all objects in the current folder
      *
+     * @deprecated
+     *
      * @return array  All object data arrays
      */
     function getObjects()
@@ -132,12 +151,14 @@ class Kolab {
      * Returns a list of all IMAP folders (including their groupware type)
      * that the current user has acccess to.
      *
+     * @deprecated
+     *
      * @return array  An array of array($foldername, $foldertype) items (empty
      *                on error).
      */
     function listFolders()
     {
-        return Kolab_IMAP::listFolders();
+        return Kolab_Storage_Deprecated::listFolders();
     }
 
     /**
@@ -312,7 +333,12 @@ class Kolab {
      */
     function getMailboxType($mailbox)
     {
-        return Kolab_IMAP::getMailboxType($mailbox);
+        $list = &Kolab_List::singleton();
+        $folder = $list->getFolder($mailbox);
+        if (is_a($folder, 'PEAR_Error')) {
+            return $folder;
+        }
+        return $folder->getType();
     }
 
     /**
@@ -586,7 +612,7 @@ class Kolab {
      */
     function encodeDate($date = false)
     {
-        return Kolab_Date::encodeDate($date);
+        return Horde_Kolab_Format_Date::encodeDate($date);
     }
 
     /**
@@ -601,7 +627,7 @@ class Kolab {
      */
     function decodeDate($date)
     {
-        return Kolab_Date::decodeDate($date);
+        return Horde_Kolab_Format_Date::decodeDate($date);
     }
 
     /**
@@ -616,7 +642,7 @@ class Kolab {
      */
     function encodeDateTime($datetime = false)
     {
-        return Kolab_Date::encodeDateTime($datetime);
+        return Horde_Kolab_Format_Date::encodeDateTime($datetime);
     }
 
     /**
@@ -631,7 +657,7 @@ class Kolab {
      */
     function decodeDateTime($datetime)
     {
-        return Kolab_Date::decodeDateTime($datetime);
+        return Horde_Kolab_Format_Date::decodeDateTime($datetime);
     }
 
     /**
@@ -647,7 +673,7 @@ class Kolab {
      */
     function decodeDateOrDateTime($date)
     {
-        return Kolab_Date::decodeDateOrDateTime($date);
+        return Horde_Kolab_Format_Date::decodeDateOrDateTime($date);
     }
 
     /**
@@ -685,7 +711,7 @@ class Kolab {
      * Returns an array of application-specific constants, that are used in
      * a generic manner throughout the library.
      *
-     * FIXME: Move to IMAP.php
+     * @deprecated
      *
      * @param string $app  The application whose constants to query.
      *
@@ -768,118 +794,22 @@ class Kolab {
     {
         global $conf;
 
-        if (!empty($conf['kolab']['misc']['multidomain']) &&
-            !empty($_SESSION['kolabHomeserver'])) {
-            // in multidomain mode we return the kolabHomeserver
-            return $_SESSION['kolabHomeserver'];
-        } else {
-            switch ($server_type) {
-            case 'imap':
-                return $conf['kolab']['imap']['server'];
-            case 'ldap':
-                return $conf['kolab']['ldap']['server'];
-            case 'smtp':
-                return $conf['kolab']['smtp']['server'];
-            default:
-                return '';
-            }
+        switch ($server_type) {
+        case 'imap':
+            return $conf['kolab']['imap']['server'];
+        case 'ldap':
+            return $conf['kolab']['ldap']['server'];
+        case 'smtp':
+            return $conf['kolab']['smtp']['server'];
+        default:
+            return '';
         }
     }
 
     /**
-     * Triggers the freebusy update for the given share uid.
-     *
-     * @return boolean True if update was successfull, false otherwise.
+     * @deprecated
      */
-    function triggerFreeBusyUpdate($share_uid)
+    function triggerFreeBusyUpdate()
     {
-        global $conf;
-
-        $folder_path = rawurldecode($share_uid);
-        if (!preg_match(";(shared\.|INBOX[/]?|user/([^/]+)/)([^@]+)(@.*)?;", $folder_path, $matches)) {
-            return PEAR::raiseError(sprintf(_("Owner of folder %s cannot be determined."), $folder_path));
-        }
-
-        if (substr($matches[1], 0, 6) == 'INBOX/') {
-            $owner =  Auth::getAuth();
-        } elseif (substr($matches[1], 0, 5) == 'user/') {
-            $domain = strstr(Auth::getAuth(), '@');
-            $user_domain = isset($matches[4]) ? $matches[4] : $domain;
-            $owner = $matches[2] . $user_domain;
-        } elseif ($matches[1] == 'shared.') {
-            return PEAR::raiseError(sprintf(_("Cannot trigger shared folder %s."), $folder_path));
-        }
-        $folder = isset($matches[3]) ? $matches[3] : '';
-
-        $url = 'https://' . Kolab::getServer("imap") .
-            '/freebusy/trigger/' . $owner . '/' . $folder . '.pfb';
-
-        // now start the request
-        $options['method'] = 'GET';
-        $options['timeout'] = 5;
-        $options['allowRedirects'] = true;
-
-        if (isset($conf['http']['proxy']) && !empty($conf['http']['proxy']['proxy_host'])) {
-            $options = array_merge($options, $conf['http']['proxy']);
-        }
-        require_once 'HTTP/Request.php';
-        $http = new HTTP_Request($url, $options);
-        $http->setBasicAuth(Auth::getAuth(), Auth::getCredential('password'));
-        @$http->sendRequest();
-        if ($http->getResponseCode() != 200) {
-            return PEAR::raiseError(sprintf(_("Unable to trigger free/busy update for folder %s on URL %s"),
-                                            $share_uid, $url));
-        }
-
-        return true;
     }
-
-    /**
-     * Rewrites a given mail or UID into the unique mail address of the user.
-     *
-     * @param string $alias  Mail or UID of the user.
-     * @return string  The mail address for a given mail or uid.
-     */
-    function kolabMailForAlias($alias)
-    {
-        // Do not try to rewrite the alias if we do not have LDAP support
-        if (!function_exists('ldap_connect')) {
-            return $alias;
-        }
-        
-        require_once 'Horde/Kolab/LDAP.php';
-
-        $kolab_ldap =& new Horde_Kolab_LDAP;
-        if (is_a($kolab_ldap, 'PEAR_Error')) {
-            return $result;
-        }
-
-        $result = $kolab_ldap->bind();
-        if (is_a($result, 'PEAR_Error')) {
-            return $result;
-        }
-
-        $dn = $kolab_ldap->dnForMail($alias);
-        if (is_a($dn, 'PEAR_Error')) {
-            return $dn;
-        }
-        if (!$dn) {
-            $dn = $kolab_ldap->dnForUid($alias);
-        }
-        if (!$dn) {
-            return PEAR::raiseError(sprintf(_("User name %s is no kolab user!"),
-                                            $alias));
-        }
-
-        $mail = $kolab_ldap->mailForDn($dn);
-        $kolab_ldap->close();
-
-        // If there is no mail attribute, return the alias instead of the error
-        if (is_a($mail, 'PEAR_Error')) {
-            return $alias;
-        }
-
-        return $mail;
-    }
-
 }
