@@ -1,8 +1,8 @@
 <?php
 /**
- * @package Horde_Kolab
+ * @package Kolab_Storage
  *
- * $Horde: framework/Kolab/Kolab/IMAP/test.php,v 1.3.2.2 2008/06/13 10:07:12 wrobel Exp $
+ * $Horde: framework/Kolab_Server/lib/Horde/Kolab/IMAP/test.php,v 1.1.2.4 2009/03/08 05:30:12 wrobel Exp $
  */
 
 /**
@@ -11,20 +11,20 @@
 define('KOLAB_IMAP_FLAG_DELETED', 1);
 
 /**
- * The Kolab_IMAP_Connection_test class simulates an IMAP server for
+ * The Horde_Kolab_IMAP_Connection_test class simulates an IMAP server for
  * testing purposes.
  *
- * $Horde: framework/Kolab/Kolab/IMAP/test.php,v 1.3.2.2 2008/06/13 10:07:12 wrobel Exp $
+ * $Horde: framework/Kolab_Server/lib/Horde/Kolab/IMAP/test.php,v 1.1.2.4 2009/03/08 05:30:12 wrobel Exp $
  *
- * Copyright 2007-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2007-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Gunnar Wrobel <wrobel@pardus.de>
- * @package Horde_Kolab
+ * @package Kolab_Storage
  */
-class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
+class Horde_Kolab_IMAP_test extends Horde_Kolab_IMAP {
 
     /**
      * If we are supposed to be connected this holds the user
@@ -75,6 +75,8 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
         $tls = ($tls) ? 'tls' : 'notls';
         $this->_connected = $login . ':' . $password . ':' . $tls;
         $this->_user = $login;
+        unset($this->_mbox);
+        $this->_mboxname = null;
     }
 
     /**
@@ -88,6 +90,15 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
         $this->_connected = null;
     }
 
+    function _parseFolder($folder)
+    {
+        if (substr($folder, 0, 5) == 'INBOX') {
+            $user = split('@', $this->_user);
+            return 'user/' . $user[0] . substr($folder, 5);
+        }
+        return $folder;
+    }
+
     /**
      * Opens the given folder.
      *
@@ -98,6 +109,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function select($folder)
     {
+        $folder = $this->_parseFolder($folder);
         if (!isset($GLOBALS['KOLAB_TESTING'][$folder])) {
             return PEAR::raiseError(sprintf("IMAP folder %s does not exist!", $folder));
         }
@@ -115,6 +127,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function exists($folder)
     {
+        $folder = $this->_parseFolder($folder);
         if (!isset($GLOBALS['KOLAB_TESTING'][$folder])) {
             return false;
         }
@@ -131,6 +144,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function create($folder)
     {
+        $folder = $this->_parseFolder($folder);
         if (isset($GLOBALS['KOLAB_TESTING'][$folder])) {
             return PEAR::raiseError(sprintf("IMAP folder %s does already exist!", $folder));
         }
@@ -142,7 +156,6 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
             'permissions' => array(),
             'annotations' => array(),
         );
-        $this->resetListCache();
         return true;
     }
 
@@ -156,11 +169,11 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function delete($folder)
     {
+        $folder = $this->_parseFolder($folder);
         if (!isset($GLOBALS['KOLAB_TESTING'][$folder])) {
             return PEAR::raiseError(sprintf("IMAP folder %s does not exist!", $folder));
         }
         unset($GLOBALS['KOLAB_TESTING'][$folder]);
-        $this->resetListCache();
         return true;
     }
 
@@ -175,6 +188,9 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function rename($old, $new)
     {
+        $old = $this->_parseFolder($old);
+        $new = $this->_parseFolder($new);
+
         if (!isset($GLOBALS['KOLAB_TESTING'][$old])) {
             return PEAR::raiseError(sprintf("IMAP folder %s does not exist!", $old));
         }
@@ -183,7 +199,6 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
         }
         $GLOBALS['KOLAB_TESTING'][$new] = $GLOBALS['KOLAB_TESTING'][$old];
         unset($GLOBALS['KOLAB_TESTING'][$old]);
-        $this->resetListCache();
         return true;
     }
 
@@ -330,7 +345,14 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function getMailboxes()
     {
-        return array_keys($GLOBALS['KOLAB_TESTING']);
+        $mboxes = array_keys($GLOBALS['KOLAB_TESTING']);
+        $user = split('@', $this->_user);
+        $pattern = '#^user/' . $user[0] . '#';
+        $result = array();
+        foreach ($mboxes as $mbox) {
+            $result[] = preg_replace($pattern, 'INBOX', $mbox);
+        }
+        return $result;
     }
 
     /**
@@ -344,6 +366,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function getAnnotation($entries, $value, $mailbox_name)
     {
+        $mailbox_name = $this->_parseFolder($mailbox_name);
         $old_mbox = null;
         if ($mailbox_name != $this->_mboxname) {
             $old_mbox = $this->_mboxname;
@@ -358,10 +381,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
         }
         $annotation = $this->_mbox['annotations'][$entries][$value];
         if ($old_mbox) {
-            $result = $this->select($old_mbox);
-            if (is_a($result, 'PEAR_Error')) {
-                return $result;
-            }
+            $this->select($old_mbox);
         }
         return $annotation;
     }
@@ -377,6 +397,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function setAnnotation($entries, $values, $mailbox_name)
     {
+        $mailbox_name = $this->_parseFolder($mailbox_name);
         $old_mbox = null;
         if ($mailbox_name != $this->_mboxname) {
             $old_mbox = $this->_mboxname;
@@ -389,7 +410,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
             $this->_mbox['annotations'][$entries] = array();
         }
         foreach ($values as $key => $value) {
-            $annotation = $this->_mbox['annotations'][$entries][$key] = $value;
+            $this->_mbox['annotations'][$entries][$key] = $value;
         }
         if ($old_mbox) {
             $result = $this->select($old_mbox);
@@ -410,6 +431,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function getACL($folder)
     {
+        $folder = $this->_parseFolder($folder);
         $old_mbox = null;
         if ($folder != $this->_mboxname) {
             $old_mbox = $this->_mboxname;
@@ -438,6 +460,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function getMyRights($folder)
     {
+        $folder = $this->_parseFolder($folder);
         $old_mbox = null;
         if ($folder != $this->_mboxname) {
             $old_mbox = $this->_mboxname;
@@ -458,7 +481,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
         }
         return $acl;
     }
-    
+
     /**
      * Set the access rights for a folder
      *
@@ -470,6 +493,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function setACL($folder, $user, $acl)
     {
+        $folder = $this->_parseFolder($folder);
         $old_mbox = null;
         if ($folder != $this->_mboxname) {
             $old_mbox = $this->_mboxname;
@@ -498,6 +522,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function deleteACL($folder, $user)
     {
+        $folder = $this->_parseFolder($folder);
         $old_mbox = null;
         if ($folder != $this->_mboxname) {
             $old_mbox = $this->_mboxname;
@@ -525,7 +550,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function appendMessage($msg)
     {
-        $split = strpos('\r\n\r\n', $msg);
+        $split = strpos($msg, "\r\n\r\n");
         $mail = array('header' => substr($msg, 0, $split + 2),
                       'body' => substr($msg, $split + 3));
         return $this->_appendMessage($mail);
@@ -547,7 +572,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
         $mail['flags'] = 0;
         $mail['header'] = $msg['header'];
         $mail['body'] = $msg['body'];
-        
+
 
         $this->_mbox['mails'][$this->_mbox['status']['uidnext']] = $mail;
         $this->_mbox['status']['uidnext']++;
@@ -564,6 +589,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function copyMessage($uid, $new_folder)
     {
+        $new_folder = $this->_parseFolder($new_folder);
         if (!$this->_mbox) {
             return PEAR::raiseError("No IMAP folder selected!");
         }
@@ -597,6 +623,7 @@ class Kolab_IMAP_Connection_test extends Kolab_IMAP_Connection {
      */
     function moveMessage($uid, $new_folder)
     {
+        $new_folder = $this->_parseFolder($new_folder);
         if (!$this->_mbox) {
             return PEAR::raiseError("No IMAP folder selected!");
         }
