@@ -47,9 +47,9 @@ define('MIME_STRING', 2);
 /**
  * The MIME:: class provides methods for dealing with MIME standards.
  *
- * $Horde: framework/MIME/MIME.php,v 1.139.4.34 2008/06/11 15:50:26 chuck Exp $
+ * $Horde: framework/MIME/MIME.php,v 1.139.4.48 2009/10/11 22:27:34 slusarz Exp $
  *
- * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -99,15 +99,22 @@ class MIME {
     var $rfc822_filter = "()<>@,;:\\\"[]\1\2\3\4\5\6\7\10\11\12\13\14\15\16\17\20\21\22\23\24\25\26\27\30\31\32\33\34\35\36\37\177";
 
     /**
-     * Determines if a string contains 8-bit characters.
+     * Determines if a string contains 8-bit (non US-ASCII) characters.
      *
-     * @param string $string  The string to check.
+     * @param string $string   The string to check.
+     * @param string $charset  The charset of the string. Defaults to
+     *                         US-ASCII. Since Horde 3.2.2.
      *
      * @return boolean  True if it does, false if it doesn't.
      */
-    function is8bit($string)
+    function is8bit($string, $charset = null)
     {
-        return (is_string($string) && preg_match('/[\x80-\xff]/', $string));
+        /* ISO-2022-JP is a 7bit charset, but it is an 8bit representation so
+         * it needs to be entirely encoded. */
+        return is_string($string) &&
+               ((stristr('iso-2022-jp', $charset) &&
+                (strstr($string, "\x1b\$B"))) ||
+                preg_match('/[\x80-\xff]/', $string));
     }
 
     /**
@@ -121,25 +128,26 @@ class MIME {
      */
     function encode($text, $charset = null)
     {
-        /* Return if nothing needs to be encoded. */
-        if (!MIME::is8bit($text)) {
-            return $text;
-        }
-
         if (is_null($charset)) {
+            require_once 'Horde/NLS.php';
             $charset = NLS::getCharset();
         }
-
         $charset = String::lower($charset);
+
         $line = '';
 
+        /* Return if nothing needs to be encoded. */
+        if (($charset == 'us-ascii') || !MIME::is8bit($text, $charset)) {
+            return $text;
+        }
+
         /* Get the list of elements in the string. */
         $size = preg_match_all('/([^\s]+)([\s]*)/', $text, $matches, PREG_SET_ORDER);
 
         foreach ($matches as $key => $val) {
-            if (MIME::is8bit($val[1])) {
+            if (MIME::is8bit($val[1], $charset)) {
                 if ((($key + 1) < $size) &&
-                    MIME::is8bit($matches[$key + 1][1])) {
+                    MIME::is8bit($matches[$key + 1][1], $charset)) {
                     $line .= MIME::_encode($val[1] . $val[2], $charset) . ' ';
                 } else {
                     $line .= MIME::_encode($val[1], $charset) . $val[2];
@@ -233,7 +241,7 @@ class MIME {
 
             /* Lines must be 76 characters or less. */
             $curr_length += $char_len;
-            if ($curr_length > 76) {
+            if ($curr_length > 75) {
                 $output .= '=' . $eol;
                 $curr_length = $char_len;
             }
@@ -278,29 +286,25 @@ class MIME {
         $text = '';
         if (is_array($addr_arr)) {
             foreach ($addr_arr as $addr) {
-                if (strlen($text) != 0) {
-                    $text .= ', ';
-                }
                 // Check for groups.
                 if (!empty($addr->groupname)) {
-                    $text .= MIME::encode($addr->groupname, $charset) . ': ' . MIME::encodeAddress($addr->addresses) . ';';
-                    continue;
-                }
-
-                if (empty($addr->personal)) {
-                    $personal = '';
+                    $text .= MIME::encode($addr->groupname, $charset) . ': ' . MIME::encodeAddress($addr->addresses) . '; ';
                 } else {
-                    if ((substr($addr->personal, 0, 1) == '"') &&
-                        (substr($addr->personal, -1) == '"')) {
-                        $addr->personal = substr($addr->personal, 1, -1);
+                    if (empty($addr->personal)) {
+                        $personal = '';
+                    } else {
+                        if ((substr($addr->personal, 0, 1) == '"') &&
+                            (substr($addr->personal, -1) == '"')) {
+                            $addr->personal = stripslashes(substr($addr->personal, 1, -1));
+                        }
+                        $personal = MIME::encode($addr->personal, $charset);
                     }
-                    $personal = MIME::encode($addr->personal, $charset);
+                    $text .= MIME::trimEmailAddress(MIME::rfc822WriteAddress($addr->mailbox, $addr->host, $personal)) . ', ';
                 }
-                $text .= MIME::trimEmailAddress(MIME::rfc822WriteAddress($addr->mailbox, $addr->host, $personal));
             }
         }
 
-        return $text;
+        return rtrim($text, ' ,');
     }
 
     /**
@@ -350,6 +354,7 @@ class MIME {
         $rest = substr($string, (strlen($preceding . $charset . $encoding . $encoded_text) + 6));
 
         if (is_null($to_charset)) {
+            require_once 'Horde/NLS.php';
             $to_charset = NLS::getCharset();
         }
 
@@ -410,7 +415,7 @@ class MIME {
         $encode = $wrap = false;
         $output = array();
 
-        if (MIME::is8bit($string)) {
+        if (MIME::is8bit($string, $charset)) {
             $string = String::lower($charset) . '\'' . (($lang === null) ? '' : String::lower($lang)) . '\'' . rawurlencode($string);
             $encode = true;
         }
@@ -466,6 +471,7 @@ class MIME {
         }
 
         if (!isset($to_charset)) {
+            require_once 'Horde/NLS.php';
             $to_charset = NLS::getCharset();
         }
 
@@ -539,19 +545,19 @@ class MIME {
     {
         $address = '';
 
-        if (!empty($personal)) {
-            $address .= MIME::_rfc822Encode($personal);
+        if (strlen($personal)) {
+            $address .= MIME::_rfc822Encode($personal, 'personal');
             $address .= ' <';
         }
 
         if (!is_null($host)) {
-            $address .= MIME::_rfc822Encode($mailbox);
+            $address .= MIME::_rfc822Encode($mailbox, 'address');
             if (substr($host, 0, 1) != '@') {
                 $address .= '@' . $host;
             }
         }
 
-        if (!empty($personal)) {
+        if (strlen($personal)) {
             $address .= '>';
         }
 
@@ -602,13 +608,14 @@ class MIME {
                     $pos = $i + 1;
                     $in_group = false;
                 }
-            } elseif ($char == ':') {
-                $in_group = true;
-            } elseif (strpos($delimiters, $char) !== false &&
-                      $prev !== '\\' &&
-                      !$in_quote) {
-                $emails[] = substr($string, $pos, $i - $pos);
-                $pos = $i + 1;
+            } elseif (!$in_quote) {
+                if ($char == ':') {
+                    $in_group = true;
+                } elseif (strpos($delimiters, $char) !== false &&
+                          $prev !== '\\') {
+                    $emails[] = substr($string, $pos, $i - $pos);
+                    $pos = $i + 1;
+                }
             }
             $prev = $char;
         }
@@ -770,6 +777,10 @@ class MIME {
      */
     function parseAddressList($address, $defserver = null, $validate = false)
     {
+        if (preg_match('/undisclosed-recipients:\s*;/i', trim($address))) {
+            return array();
+        }
+
         /* Use built-in IMAP function only if available and if not parsing
          * distribution lists because it doesn't parse distribution lists
          * properly. */
@@ -783,32 +794,50 @@ class MIME {
                 require_once 'Mail/RFC822.php';
                 $parser = new Mail_RFC822();
             }
-            return $parser->parseAddressList($address, $defserver, false,
+            $ret = $parser->parseAddressList($address, $defserver, false,
                                              $validate);
+            return is_a($ret, 'PEAR_Error') ? array() : $ret;
         }
     }
 
     /**
-     * Quotes and escapes the given string if necessary.
+     * Quotes and escapes the given string if necessary using rules contained
+     * in RFC 2822 [3.2.5].
      *
      * @access private
      *
-     * @param string $str     The string to be quoted and escaped.
-     * @param string $filter  A list of characters that make it necessary to
-     *                        quote the string if they occur.
+     * @param string $str  The string to be quoted and escaped.
+     * @param string $type  Either 'address' or 'personal'.
      *
      * @return string  The correctly quoted and escaped string.
      */
-    function _rfc822Encode($str, $filter = '')
+    function _rfc822Encode($str, $type = 'address')
     {
-        if (empty($filter)) {
-            $vars = get_class_vars('MIME');
-            $filter = $vars['rfc822_filter'] . ' ';
+        // Excluded (in ASCII): 0-8, 10-31, 34, 40-41, 44, 58-60, 62, 64,
+        // 91-93, 127
+        $filter = "\0\1\2\3\4\5\6\7\10\12\13\14\15\16\17\20\21\22\23\24\25\26\27\30\31\32\33\34\35\36\37\"(),:;<>@[\\]\177";
+
+        switch ($type) {
+        case 'address':
+            // RFC 2822 [3.4.1]: (HTAB, SPACE) not allowed in address
+            $filter .= "\11\40";
+            break;
+
+        case 'personal':
+            // RFC 2822 [3.4]: Period not allowed in display name
+            $filter .= '.';
+            break;
+
+        default:
+            // BC: $filter was passed in explicitly
+            $filter = $type;
         }
 
         // Strip double quotes if they are around the string already.
+        // If quoted, we know that the contents are already escaped, so
+        // unescape now.
         if (substr($str, 0, 1) == '"' && substr($str, -1) == '"') {
-            $str = substr($str, 1, -1);
+            $str = stripslashes(substr($str, 1, -1));
         }
 
         if (strcspn($str, $filter) != strlen($str)) {
@@ -930,10 +959,10 @@ class MIME {
         if (($header_lower != 'content-type') &&
             ($header_lower != 'content-disposition')) {
             /* Wrap the line. */
-            $line = wordwrap($text, 75, $eol . "\t");
+            $line = wordwrap($text, 75, $eol . ' ');
 
             /* Make sure there are no empty lines. */
-            $line = preg_replace('/' . $eol . "\t\s*" . $eol . "\t/", '/' . $eol . "\t/", $line);
+            $line = preg_replace('/' . $eol . ' ' . $eol . ' /', '/' . $eol . " /", $line);
 
             return substr($line, strlen($header) + 2);
         }
@@ -962,7 +991,7 @@ class MIME {
             }
 
             if (!empty($line)) {
-                $line .= "\t";
+                $line .= ' ';
             }
             $line .= $val . (($moreparams) ? ';' : '') . $eol;
         }
