@@ -40,9 +40,9 @@ define('MIME_DEFAULT_ENCODING', '7bit');
  * The MIME_Part:: class provides a wrapper around MIME parts and methods
  * for dealing with them.
  *
- * $Horde: framework/MIME/MIME/Part.php,v 1.177.4.23 2008/01/02 11:30:39 jan Exp $
+ * $Horde: framework/MIME/MIME/Part.php,v 1.177.4.29 2010/04/13 12:12:08 jan Exp $
  *
- * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -872,7 +872,6 @@ class MIME_Part {
     {
         $eol = $this->getEOL();
         $ptype = $this->getPrimaryType();
-        $stype = $this->getSubType();
 
         /* Get the character set for this part. */
         $charset = $this->getCharset();
@@ -891,7 +890,8 @@ class MIME_Part {
              * parameter.  See:
              *   http://lists.horde.org/archives/dev/Week-of-Mon-20040426/014240.html */
             if (!empty($GLOBALS['conf']['mailformat']['brokenrfc2231']) &&
-                (strpos($encode_2231, '*=') !== false)) {
+                ((strpos($encode_2231, '*=') !== false) ||
+                 (strpos($encode_2231, '*0=') !== false))) {
                 $ctype .= '; ' . $key . '="' . MIME::encode($value, $charset) . '"';
             }
             $ctype .= '; ' . $encode_2231;
@@ -903,15 +903,6 @@ class MIME_Part {
             $headers['Content-Description'] = MIME::wrapHeaders('Content-Description', MIME::encode($descrip, $charset), $eol);
         }
 
-        /* RFC 2045 [4] - message/rfc822 and message/partial require the
-           MIME-Version header only if they themselves claim to be MIME
-           compliant. */
-        if (($ptype == 'message') &&
-            (($stype == 'rfc822') || ($stype == 'partial')) &&
-            (strpos($this->_contents, 'MIME-Version: 1.0') !== false)) {
-            $headers['MIME-Version'] = '1.0';
-        }
-
         /* message/* parts require no additional header information. */
         if ($ptype == 'message') {
             return $headers;
@@ -928,7 +919,8 @@ class MIME_Part {
                 $encode_2231 = MIME::encodeRFC2231('filename', $name, $charset);
                 /* Same broken RFC 2231 workaround as above. */
                 if (!empty($GLOBALS['conf']['mailformat']['brokenrfc2231']) &&
-                    (strpos($encode_2231, '*=') !== false)) {
+                    ((strpos($encode_2231, '*=') !== false) ||
+                     (strpos($encode_2231, '*0=') !== false))) {
                     $disp .= '; filename="' . MIME::encode($name, $charset) . '"';
                 }
                 $disp .= '; ' . $encode_2231;
@@ -942,7 +934,7 @@ class MIME_Part {
 
         /* Add content ID information. */
         if (!is_null($this->_contentid)) {
-            $headers['Content-ID'] = $this->_contentid;
+            $headers['Content-ID'] = '<' . $this->_contentid . '>';
         }
 
         return $headers;
@@ -1059,9 +1051,10 @@ class MIME_Part {
             break;
 
         case 'text':
+            $eol = $this->getEOL();
             if (MIME::is8bit($text)) {
                 $encoding = ($this->_encode7bit) ? 'quoted-printable' : '8bit';
-            } elseif (preg_match("/(?:\n|^)[^\n]{999,}(?:\n|$)/", $text)) {
+            } elseif (preg_match("/(?:" . $eol . "|^)[^" . $eol . "]{999,}(?:" . $eol . "|$)/", $this->_contents)) {
                 /* If the text is longer than 998 characters between
                  * linebreaks, use quoted-printable encoding to ensure the
                  * text will not be chopped (i.e. by sendmail if being sent
@@ -1352,8 +1345,7 @@ class MIME_Part {
             return $bytes;
         }
 
-        $localeinfo = NLS::getLocaleInfo();
-        return number_format($bytes / 1024, 2, $localeinfo['decimal_point'], $localeinfo['thousands_sep']);
+        return NLS::numberFormat($bytes / 1024, 2);
      }
 
     /**
