@@ -1,6 +1,6 @@
 <?php
 /**
- * $Horde: framework/RPC/RPC/webdav.php,v 1.1.12.17 2008/06/13 15:02:19 chuck Exp $
+ * $Horde: framework/RPC/RPC/webdav.php,v 1.1.12.19 2009/01/06 15:23:32 jan Exp $
  *
  * @package Horde_RPC
  */
@@ -12,7 +12,7 @@ include_once 'HTTP/WebDAV/Server.php';
  * The Horde_RPC_webdav class provides a WebDAV implementation of the
  * Horde RPC system.
  *
- * Copyright 2004-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2004-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -363,7 +363,7 @@ class HTTP_WebDAV_Server_Horde extends HTTP_WebDAV_Server {
             }
             if (empty($items)) {
                 // No content exists at this level.
-                return false;
+                return array();
             }
             if (!is_array(reset($items))) {
                 /* A one-dimensional array means we have an actual object with
