@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: framework/Horde/Horde/Registry.php,v 1.243.2.36 2008/05/21 22:39:14 jan Exp $
+ * $Horde: framework/Horde/Horde/Registry.php,v 1.243.2.39 2009/01/06 15:23:10 jan Exp $
  *
- * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -323,7 +323,7 @@ class Registry {
                 $_SESSION['_registry'] = array('cache' => array(), 'mtime' => false);
             }
 
-            foreach (array_unique($this->_updatecache) as $val) {
+            foreach (array_keys(array_flip($this->_updatecache)) as $val) {
                 $_SESSION['_registry']['cache'][$val] = $this->$val;
             }
         }
@@ -425,17 +425,13 @@ class Registry {
      */
     function listAPIs()
     {
-        if (!empty($this->_apis)) {
-            return $this->_apis;
+        if (empty($this->_apis)) {
+            foreach (array_keys($this->_interfaces) as $interface) {
+                list($api,) = explode('/', $interface, 2);
+                $this->_apis[$api] = true;
+            }
         }
-
-        foreach (array_keys($this->_interfaces) as $interface) {
-            list($api,) = explode('/', $interface, 2);
-            $this->_apis[] = $api;
-        }
-
-        $this->_apis = array_unique($this->_apis);
-        return $this->_apis;
+        return array_keys($this->_apis);
     }
 
     /**
@@ -467,19 +463,19 @@ class Registry {
                 if (strpos($method, '/') !== false) {
                     if (is_null($api) ||
                         (substr($method, 0, strlen($api)) == $api)) {
-                        $methods[] = $method;
+                        $methods[$method] = true;
                     }
                 } elseif (is_null($api) || ($method == $api)) {
                     if (isset($this->_apiCache[$app])) {
                         foreach (array_keys($this->_apiCache[$app]) as $service) {
-                            $methods[] = $method . '/' . $service;
+                            $methods[$method . '/' . $service] = true;
                         }
                     }
                 }
             }
         }
 
-        return array_unique($methods);
+        return array_keys($methods);
     }
 
     /**
@@ -819,6 +815,14 @@ class Registry {
             return PEAR::raiseError(sprintf(_('%s is not authorised for %s.'), Auth::getAuth() ? 'User ' . Auth::getAuth() : 'Guest user', $this->applications[$app]['name']), 'permission_denied');
         }
 
+        /* Chicken and egg problem: the language environment has to be loaded
+         * before loading the configuration file, because it might contain
+         * gettext strings. Though the preferences can specify a different
+         * language for this app, the have to be loaded after the
+         * configuration, because they rely on configuration settings. So try
+         * with the current language, and reset the language later. */
+        NLS::setLanguageEnvironment($GLOBALS['language'], $app);
+
         /* Import this application's configuration values. */
         $success = $this->importConfig($app);
         if (is_a($success, 'PEAR_Error')) {
@@ -834,8 +838,10 @@ class Registry {
         $language = '';
         if (isset($GLOBALS['prefs'])) {
             $language = $GLOBALS['prefs']->getValue('language');
+            if ($language != $GLOBALS['language']) {
+                NLS::setLanguageEnvironment($language, $app);
+            }
         }
-        NLS::setLanguageEnvironment($language, $app);
 
         /* Once we know everything succeeded and is in a consistent state
          * again, push the new application onto the stack. */
