@@ -3,9 +3,9 @@
  * Horde_Share_sql:: provides the sql backend for the horde share
  * driver.
  *
- * $Horde: framework/Share/Share/sql.php,v 1.1.2.33 2008/06/09 23:56:54 chuck Exp $
+ * $Horde: framework/Share/Share/sql.php,v 1.1.2.58 2010/03/17 16:20:15 jan Exp $
  *
- * Copyright 2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2008-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -33,11 +33,6 @@ class Horde_Share_sql extends Horde_Share {
      */
     var $_db;
 
-    /**
-     * SQL connection parameters
-     */
-    var $_params = array();
-
     /**
      * Handle for the current database connection, used for writing. Defaults
      * to the same handle as $db if a separate write database is not required.
@@ -46,6 +41,11 @@ class Horde_Share_sql extends Horde_Share {
      */
     var $_write_db;
 
+    /**
+     * SQL connection parameters
+     */
+    var $_params = array();
+
     /**
      * Main share table for the current scope.
      *
@@ -75,6 +75,20 @@ class Horde_Share_sql extends Horde_Share {
         parent::__wakeup();
     }
 
+    /**
+     * Returns the properties that need to be serialized.
+     *
+     * @return array  List of serializable properties.
+     */
+    function __sleep()
+    {
+        $properties = get_object_vars($this);
+        unset($properties['_sortList'],
+              $properties['_db'],
+              $properties['_write_db']);
+        return array_keys($properties);
+    }
+
     /**
      * Get storage table
      */
@@ -115,8 +129,12 @@ class Horde_Share_sql extends Horde_Share {
     function _getShareUsers(&$share)
     {
         if ($this->_hasUsers($share)) {
-            $query = $this->_db->prepare('SELECT user_uid, perm FROM ' . $this->_table . '_users WHERE share_id = ?');
-            $result = $query->execute(array($share['share_id']));
+            $stmt = $this->_db->prepare('SELECT user_uid, perm FROM ' . $this->_table . '_users WHERE share_id = ?');
+            if (is_a($stmt, 'PEAR_Error')) {
+                Horde::logMessage($stmt, __FILE__, __LINE__, PEAR_LOG_ERR);
+                return $stmt;
+            }
+            $result = $stmt->execute(array($share['share_id']));
             if (is_a($result, 'PEAR_Error')) {
                 Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
                 return $result;
@@ -125,9 +143,10 @@ class Horde_Share_sql extends Horde_Share {
                     $share['perm']['users'][$row['user_uid']] = (int)$row['perm'];
                 }
             }
+            $stmt->free();
+            $result->free();
         }
     }
-
     /**
      * Get groups permissions
      *
@@ -137,16 +156,22 @@ class Horde_Share_sql extends Horde_Share {
     {
         if ($this->_hasGroups($share)) {
             // Get groups permissions
-            $query = $this->_db->prepare('SELECT group_uid, perm FROM ' . $this->_table . '_groups WHERE share_id = ?');
-            $result = $query->execute(array($share['share_id']));
+            $stmt = $this->_db->prepare('SELECT group_uid, perm FROM ' . $this->_table . '_groups WHERE share_id = ?');
+            if (is_a($stmt, 'PEAR_Error')) {
+                Horde::logMessage($stmt, __FILE__, __LINE__, PEAR_LOG_ERR);
+                return $stmt;
+            }
+            $result = $stmt->execute(array($share['share_id']));
             if (is_a($result, 'PEAR_Error')) {
                 Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
                 return $result;
             } elseif (!empty($result)) {
                 while ($row = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
-                    $share['perm']['groups'][(int)$row['group_uid']] = (int)$row['perm'];
+                    $share['perm']['groups'][$row['group_uid']] = (int)$row['perm'];
                 }
             }
+            $stmt->free();
+            $result->free();
         }
     }
 
@@ -160,8 +185,12 @@ class Horde_Share_sql extends Horde_Share {
      */
     function &_getShare($name)
     {
-        $query = $this->_db->prepare('SELECT * FROM ' . $this->_table . ' WHERE share_name = ?');
-        $results = $query->execute(array($name));
+        $stmt = $this->_db->prepare('SELECT * FROM ' . $this->_table . ' WHERE share_name = ?');
+        if (is_a($stmt, 'PEAR_Error')) {
+            Horde::logMessage($stmt, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return $stmt;
+        }
+        $results = $stmt->execute(array($name));
         if (is_a($results, 'PEAR_Error')) {
             Horde::logMessage($results, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $results;
@@ -173,9 +202,11 @@ class Horde_Share_sql extends Horde_Share {
         } elseif (empty($data)) {
             return PEAR::RaiseError(sprintf(_("Share \"%s\" does not exist."), $name));
         }
+        $stmt->free();
+        $results->free();
 
         // Convert charset
-        $data = $this->_fromDriver($data);
+        $data = $this->_fromDriverCharset($data);
 
         // Populate the perms array
         $this->_loadPermissions($data);
@@ -215,8 +246,12 @@ class Horde_Share_sql extends Horde_Share {
     function &_getShareById($id)
     {
         $params = array($id);
-        $query = $this->_db->prepare('SELECT * FROM ' . $this->_table . ' WHERE share_id = ?');
-        $results = $query->execute($params);
+        $stmt = $this->_db->prepare('SELECT * FROM ' . $this->_table . ' WHERE share_id = ?');
+        if (is_a($stmt, 'PEAR_Error')) {
+            Horde::logMessage($stmt, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return $stmt;
+        }
+        $results = $stmt->execute($params);
         if (is_a($results, 'PEAR_Error')) {
             Horde::logMessage($results, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $results;
@@ -229,8 +264,11 @@ class Horde_Share_sql extends Horde_Share {
             return PEAR::RaiseError(sprintf(_("Share ID %d does not exist."), $id));
         }
 
+        $stmt->free();
+        $results->free();
+
         // Convert charset
-        $data = $this->_fromDriver($data);
+        $data = $this->_fromDriverCharset($data);
 
         // Get permissions
         $this->_loadPermissions($data);
@@ -263,7 +301,7 @@ class Horde_Share_sql extends Horde_Share {
         $groups = array();
         $users = array();
         while ($share = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
-            $shares[(int)$share['share_id']] = $this->_fromDriver($share);
+            $shares[(int)$share['share_id']] = $this->_fromDriverCharset($share);
             if ($this->_hasUsers($share)) {
                 $users[] = (int)$share['share_id'];
             }
@@ -271,6 +309,7 @@ class Horde_Share_sql extends Horde_Share {
                 $groups[] = (int)$share['share_id'];
             }
         }
+        $result->free();
 
         // Get users permissions
         if (!empty($users)) {
@@ -284,6 +323,7 @@ class Horde_Share_sql extends Horde_Share {
                 while ($share = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
                     $shares[$share['share_id']]['perm']['users'][$share['user_uid']] = (int)$share['perm'];
                 }
+                $result->free();
             }
         }
 
@@ -299,6 +339,7 @@ class Horde_Share_sql extends Horde_Share {
                 while ($share = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
                     $shares[$share['share_id']]['perm']['groups'][$share['group_uid']] = (int)$share['perm'];
                 }
+                $result->free();
             }
         }
 
@@ -344,8 +385,9 @@ class Horde_Share_sql extends Horde_Share {
         }
 
         while ($share = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
-            $shares[(int)$share['share_id']] = $this->_fromDriver($share);
+            $shares[(int)$share['share_id']] = $this->_fromDriverCharset($share);
         }
+        $result->free();
 
         // Get users permissions
         $query = 'SELECT share_id, user_uid, perm FROM ' . $this->_table . '_users';
@@ -357,6 +399,7 @@ class Horde_Share_sql extends Horde_Share {
             while ($share = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
                 $shares[$share['share_id']]['perm']['users'][$share['user_uid']] = (int)$share['perm'];
             }
+            $result->free();
         }
 
         // Get groups permissions
@@ -367,8 +410,9 @@ class Horde_Share_sql extends Horde_Share {
             return $result;
         } elseif (!empty($result)) {
             while ($share = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
-                $shares[$share['share_id']]['perm']['groups'][(int)$share['group_uid']] = (int)$share['perm'];
+                $shares[$share['share_id']]['perm']['groups'][$share['group_uid']] = (int)$share['perm'];
             }
+            $result->free();
         }
 
         $sharelist = array();
@@ -405,18 +449,22 @@ class Horde_Share_sql extends Horde_Share {
             $sortfield = 's.attribute_' . $sort_by;
         }
 
-        /* If we have more than a username, see if we need to convert charset */
-        if (is_array($attributes)) {
-            $attributes = $this->_toDriver($attributes);
-        }
-
-        $query = 'SELECT s.* '
+        $query = 'SELECT DISTINCT s.* '
             . $this->_getShareCriteria($userid, $perm, $attributes)
             . ' ORDER BY ' . $sortfield
             . (($direction == 0) ? ' ASC' : ' DESC');
         if ($from > 0 || $count > 0) {
             $this->_db->setLimit($count, $from);
         }
+
+        // Fix field names for sqlite. MDB2 tries to handle this with
+        // MDB2_PORTABILITY_FIX_ASSOC_FIELD_NAMES, but it doesn't stick.
+        if ($this->_db->phptype == 'sqlite') {
+            $connection = &$this->_db->getConnection();
+            @sqlite_query('PRAGMA full_column_names=0', $connection);
+            @sqlite_query('PRAGMA short_column_names=1', $connection);
+        }
+
         $result = $this->_db->query($query);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
@@ -428,7 +476,7 @@ class Horde_Share_sql extends Horde_Share {
         $users = array();
         $groups = array();
         while ($share = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
-            $shares[(int)$share['share_id']] = $this->_fromDriver($share);
+            $shares[(int)$share['share_id']] = $this->_fromDriverCharset($share);
             if ($this->_hasUsers($share)) {
                 $users[] = (int)$share['share_id'];
             }
@@ -436,6 +484,7 @@ class Horde_Share_sql extends Horde_Share {
                 $groups[] = (int)$share['share_id'];
             }
         }
+        $result->free();
 
         // Get users permissions
         if (!empty($users)) {
@@ -450,6 +499,7 @@ class Horde_Share_sql extends Horde_Share {
                 while ($share = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
                     $shares[$share['share_id']]['perm']['users'][$share['user_uid']] = (int)$share['perm'];
                 }
+                $result->free();
             }
         }
 
@@ -464,8 +514,9 @@ class Horde_Share_sql extends Horde_Share {
                 return $result;
             } elseif (!empty($result)) {
                 while ($share = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
-                    $shares[$share['share_id']]['perm']['groups'][(int)$share['group_uid']] = (int)$share['perm'];
+                    $shares[$share['share_id']]['perm']['groups'][$share['group_uid']] = (int)$share['perm'];
                 }
+                $result->free();
             }
         }
 
@@ -503,7 +554,7 @@ class Horde_Share_sql extends Horde_Share {
     function _countShares($userid, $perm = PERMS_SHOW, $attributes = null)
     {
         $query = $this->_getShareCriteria($userid, $perm, $attributes);
-        $query = 'SELECT COUNT(*) ' . $query;
+        $query = 'SELECT COUNT(DISTINCT s.share_id) ' . $query;
 
         return $this->_db->queryOne($query);
     }
@@ -547,12 +598,19 @@ class Horde_Share_sql extends Horde_Share {
                         $this->_table . '_users',
                         $this->_table . '_groups');
         foreach ($tables as $table) {
-            $query = $this->_write_db->prepare('DELETE FROM ' . $this->_table . ' WHERE share_id = ?', null, MDB2_PREPARE_MANIP);
-            $result = $query->execute($params);
+
+            /* Remove the share entry */
+            $stmt = $this->_write_db->prepare('DELETE FROM ' . $table . ' WHERE share_id = ?', null, MDB2_PREPARE_MANIP);
+            if (is_a($stmt, 'PEAR_Error')) {
+                Horde::logMessage($stmt, __FILE__, __LINE__, PEAR_LOG_ERR);
+                return $stmt;
+            }
+            $result = $stmt->execute($params);
             if (is_a($result, 'PEAR_Error')) {
                 Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
                 return $result;
             }
+            $stmt->free();
         }
 
         return true;
@@ -567,19 +625,25 @@ class Horde_Share_sql extends Horde_Share {
      */
     function _exists($share)
     {
-        $query = $this->_db->prepare('SELECT 1 FROM ' . $this->_table
-                 . ' WHERE share_name = ?');
+        $stmt = $this->_db->prepare('SELECT 1 FROM ' . $this->_table
+                . ' WHERE share_name = ?');
+
+        if (is_a($stmt, 'PEAR_Error')) {
+            Horde::logMessage($stmt, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return $stmt;
 
-        if (is_a($query, 'PEAR_Error')) {
-            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
-            return $result;
         }
-        $result = $query->execute(array($share));
+        $result = $stmt->execute(array($share));
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
         }
-        return (bool)$result->fetchOne();
+
+        $exists = (bool)$result->fetchOne();
+        $stmt->free();
+        $result->free();
+
+        return $exists;
     }
 
     /**
@@ -595,6 +659,8 @@ class Horde_Share_sql extends Horde_Share {
      */
     function _getShareCriteria($userid, $perm = PERMS_SHOW, $attributes = null)
     {
+        require_once 'Horde/SQL.php';
+
         $query = ' FROM ' . $this->_table . ' s ';
         $where = '';
 
@@ -603,15 +669,15 @@ class Horde_Share_sql extends Horde_Share {
             $where .= 's.share_owner = ' . $this->_write_db->quote($userid);
 
             // (name == perm_creator and val & $perm)
-            $where .= ' OR (s.perm_creator & ' . $perm . ') <> 0';
+            $where .= ' OR (' . Horde_SQL::buildClause($this->_db, 's.perm_creator', '&', $perm) . ')';
 
             // (name == perm_creator and val & $perm)
-            $where .= ' OR (s.perm_default & ' . $perm . ') <> 0';
+            $where .= ' OR (' . Horde_SQL::buildClause($this->_db, 's.perm_default',  '&', $perm) . ')';
 
             // (name == perm_users and key == $userid and val & $perm)
             $query .= ' LEFT JOIN ' . $this->_table . '_users AS u ON u.share_id = s.share_id';
             $where .= ' OR ( u.user_uid = ' .  $this->_write_db->quote($userid)
-                . ' AND (u.perm & ' . $perm . ') <> 0)';
+                . ' AND (' . Horde_SQL::buildClause($this->_db, 'u.perm', '&', $perm) . '))';
 
             // If the user has any group memberships, check for those also.
             require_once 'Horde/Group.php';
@@ -619,16 +685,24 @@ class Horde_Share_sql extends Horde_Share {
             $groups = $group->getGroupMemberships($userid, true);
             if (!is_a($groups, 'PEAR_Error') && $groups) {
                 // (name == perm_groups and key in ($groups) and val & $perm)
+                $ids = array_keys($groups);
+                $group_ids = array();
+                foreach ($ids as $id) {
+                    $group_ids[] = $this->_db->quote((string)$id);
+                }
                 $query .= ' LEFT JOIN ' . $this->_table . '_groups AS g ON g.share_id = s.share_id';
-                $where .= ' OR ( g.group_uid IN ("' . implode('","', array_keys($groups)) . '")'
-                    . ' AND (g.perm & ' . $perm .') <> 0)';
+                $where .= ' OR (g.group_uid IN (' . implode(',', $group_ids) . ')'
+                    . ' AND (' . Horde_SQL::buildClause($this->_db, 'g.perm', '&', $perm) . '))';
             } elseif (is_a($groups, 'PEAR_Error')) {
                 Horde::logMessage($groups, __FILE__, __LINE__, PEAR_LOG_ERR);
             }
         } else {
-            $where = '(s.perm_guest & ' . $perm . ') <> 0';
+            $where = '(' . Horde_SQL::buildClause($this->_db, 's.perm_guest', '&', $perm) . ')';
         }
 
+        $attributes = $this->_toDriverKeys($attributes);
+        $attributes = $this->_toDriverCharset($attributes);
+
         if (is_array($attributes)) {
             // Build attribute/key filter.
             $where = ' (' . $where . ') ';
@@ -644,6 +718,17 @@ class Horde_Share_sql extends Horde_Share {
         return $query . ' WHERE ' . $where;
     }
 
+    /**
+     * Resets the current database name so that MDB2 is always selecting the
+     * database before sending a query.
+     */
+    function _selectDB(&$db, $scope, $message, $is_manip = null)
+    {
+        if ($scope == 'query') {
+            $db->connected_database_name = '';
+        }
+    }
+
     /**
      * Attempts to open a connection to the sql server.
      *
@@ -666,17 +751,31 @@ class Horde_Share_sql extends Horde_Share {
         require_once 'MDB2.php';
         $params = $this->_params;
         unset($params['charset']);
-        $this->_write_db = MDB2::factory($params);
+        $this->_write_db = &MDB2::factory($params);
         if (is_a($this->_write_db, 'PEAR_Error')) {
             Horde::fatal($this->_write_db, __FILE__, __LINE__);
         }
 
+        /* Attach debug handler. */
+        $this->_write_db->setOption('debug', true);
+        $this->_write_db->setOption('seqcol_name', 'id');
+        $this->_write_db->setOption('debug_handler', array($this, '_selectDB'));
+
         /* Set DB portability options. */
         switch ($this->_write_db->phptype) {
         case 'mssql':
             $this->_write_db->setOption('field_case', CASE_LOWER);
             $this->_write_db->setOption('portability', MDB2_PORTABILITY_FIX_CASE | MDB2_PORTABILITY_ERRORS | MDB2_PORTABILITY_RTRIM | MDB2_PORTABILITY_FIX_ASSOC_FIELD_NAMES);
             break;
+
+        case 'pgsql':
+            /* The debug handler breaks PostgreSQL. In most cases it shouldn't
+             * be necessary, but this may mean we simply can't support use of
+             * multiple Postgres databases right now. See
+             * http://bugs.horde.org/ticket/7825 */
+            $this->_write_db->setOption('debug', false);
+            // Fall through
+
         default:
             $this->_write_db->setOption('field_case', CASE_LOWER);
             $this->_write_db->setOption('portability', MDB2_PORTABILITY_FIX_CASE | MDB2_PORTABILITY_ERRORS | MDB2_PORTABILITY_FIX_ASSOC_FIELD_NAMES);
@@ -685,10 +784,12 @@ class Horde_Share_sql extends Horde_Share {
         /* Check if we need to set up the read DB connection seperately. */
         if (!empty($this->_params['splitread'])) {
             $params = array_merge($params, $this->_params['read']);
-            $this->_db = MDB2::factory($params);
+            unset($params['charset']);
+            $this->_db = &MDB2::singleton($params);
             if (is_a($this->_db, 'PEAR_Error')) {
                 Horde::fatal($this->_db, __FILE__, __LINE__);
             }
+            $this->_db->setOption('seqcol_name', 'id');
 
             /* Set DB portability options. */
             switch ($this->_db->phptype) {
@@ -696,12 +797,22 @@ class Horde_Share_sql extends Horde_Share {
                 $this->_db->setOption('field_case', CASE_LOWER);
                 $this->_db->setOption('portability', MDB2_PORTABILITY_FIX_CASE | MDB2_PORTABILITY_ERRORS | MDB2_PORTABILITY_RTRIM | MDB2_PORTABILITY_FIX_ASSOC_FIELD_NAMES);
                 break;
+
+            case 'pgsql':
+                /* The debug handler breaks PostgreSQL. In most cases it shouldn't
+                 * be necessary, but this may mean we simply can't support use of
+                 * multiple Postgres databases right now. See
+                 * http://bugs.horde.org/ticket/7825 */
+                $this->_write_db->setOption('debug', false);
+                // Fall through
+
             default:
+                $this->_db->setOption('field_case', CASE_LOWER);
                 $this->_db->setOption('portability', MDB2_PORTABILITY_FIX_CASE | MDB2_PORTABILITY_ERRORS | MDB2_PORTABILITY_FIX_ASSOC_FIELD_NAMES);
             }
         } else {
             /* Default to the same DB handle as the writer for reading too */
-            $this->_db = $this->_write_db;
+            $this->_db = &$this->_write_db;
         }
 
         return true;
@@ -710,7 +821,7 @@ class Horde_Share_sql extends Horde_Share {
     /**
      * Utility function to convert from the SQL server's charset.
      */
-    function _fromDriver($data)
+    function _fromDriverCharset($data)
     {
         foreach ($data as $key => $value) {
             if (substr($key, 0, 9) == 'attribute') {
@@ -725,8 +836,12 @@ class Horde_Share_sql extends Horde_Share {
     /**
      * Utility function to convert TO the SQL server's charset.
      */
-    function _toDriver($data)
+    function _toDriverCharset($data)
     {
+        if (!is_array($data)) {
+            return $data;
+        }
+
         foreach ($data as $key => $value) {
             if (substr($key, 0, 9) == 'attribute') {
                 $data[$key] = String::convertCharset(
@@ -737,6 +852,32 @@ class Horde_Share_sql extends Horde_Share {
         return $data;
     }
 
+    /**
+     * Convert an array keyed on client keys to an array keyed on the driver
+     * keys.
+     *
+     * @param array  $data  The client code keyed array.
+     *
+     * @return array  The driver keyed array.
+     */
+    function _toDriverKeys($data)
+    {
+        if (!is_array($data)) {
+            return $data;
+        }
+
+        $driver_keys = array();
+        foreach ($data as $key => $value) {
+            if ($key == 'owner') {
+                $driver_keys['share_owner'] = $value;
+            } else {
+                $driver_keys['attribute_' . $key] = $value;
+            }
+        }
+
+        return $driver_keys;
+    }
+
 }
 
 /**
@@ -823,7 +964,7 @@ class Horde_Share_Object_sql extends Horde_Share_Object {
      */
     function _getId()
     {
-        return $this->data['share_id'];
+        return isset($this->data['share_id']) ? $this->data['share_id'] : null;
     }
 
     /**
@@ -847,7 +988,7 @@ class Horde_Share_Object_sql extends Horde_Share_Object {
         $fields = array();
         $params = array();
 
-        foreach ($this->_shareOb->_toDriver($this->data) as $key => $value) {
+        foreach ($this->_shareOb->_toDriverCharset($this->data) as $key => $value) {
             if ($key != 'share_id' && $key != 'perm' && $key != 'share_flags') {
                 $fields[] = $key;
                 $params[] = $value;
@@ -889,78 +1030,75 @@ class Horde_Share_Object_sql extends Horde_Share_Object {
             $query = 'UPDATE ' . $table . ' SET ' . implode(' = ?, ', $fields) . ' = ? WHERE share_id = ?';
             $params[] = $this->data['share_id'];
         }
-        $query = $db->prepare($query, null, MDB2_PREPARE_MANIP);
-        if (is_a($query, 'PEAR_Error')) {
-             Horde::logMessage($query, __FILE__, __LINE__, PEAR_LOG_ERR);
-            return $query;
+        $stmt = $db->prepare($query, null, MDB2_PREPARE_MANIP);
+        if (is_a($stmt, 'PEAR_Error')) {
+            Horde::logMessage($stmt, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return $stmt;
         }
-        $result = $query->execute($params);
+        $result = $stmt->execute($params);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
         }
+        $stmt->free();
 
         // Update the share's user permissions
-        $query = $db->prepare('DELETE FROM ' . $table . '_users WHERE share_id = ?', null, MDB2_PREPARE_MANIP);
-        if (is_a($query, 'PEAR_Error')) {
-            Horde::logMessage($query, __FILE__, __LINE__, PEAR_LOG_ERR);
-            return $query;
+        $stmt = $db->prepare('DELETE FROM ' . $table . '_users WHERE share_id = ?', null, MDB2_PREPARE_MANIP);
+        if (is_a($stmt, 'PEAR_Error')) {
+            Horde::logMessage($stmt, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return $stmt;
         }
-        $result = $query->execute(array($this->data['share_id']));
+        $result = $stmt->execute(array($this->data['share_id']));
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
         }
-
-        if (!empty($this->data['perm']['users']) ||
-            !empty($this->data['perm']['groups'])) {
-
-            $db->loadModule('Extended', null, false);
-        }
+        $stmt->free();
 
         if (!empty($this->data['perm']['users'])) {
             $data = array();
             foreach ($this->data['perm']['users'] as $user => $perm) {
-                $data[] = array($this->data['share_id'], $user, $perm);
-            }
-            $sth = $db->prepare('INSERT INTO ' . $table . '_users (share_id, user_uid, perm) VALUES (?, ?, ?)', null, MDB2_PREPARE_MANIP);
-            if (is_a($sth, 'PEAR_Error')) {
-                Horde::logMessage($sth, __FILE__, __LINE__, PEAR_LOG_ERR);
-                return $sth;
-            }
-            $result = $db->extended->executeMultiple($sth, $data);
-            if (is_a($result, 'PEAR_Error')) {
-                Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
-                return $result;
+                $stmt = $db->prepare('INSERT INTO ' . $table . '_users (share_id, user_uid, perm) VALUES (?, ?, ?)', null, MDB2_PREPARE_MANIP);
+                if (is_a($stmt, 'PEAR_Error')) {
+                    Horde::logMessage($stmt, __FILE__, __LINE__, PEAR_LOG_ERR);
+                    return $stmt;
+                }
+                $result = $stmt->execute(array($this->data['share_id'], $user, $perm));
+                if (is_a($result, 'PEAR_Error')) {
+                    Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+                    return $result;
+                }
+                $stmt->free();
             }
         }
 
         // Update the share's group permissions
-        $query = $db->prepare('DELETE FROM ' . $table . '_groups WHERE share_id = ?', null, MDB2_PREPARE_MANIP);
-        if (is_a($query, 'PEAR_Error')) {
-            Horde::logMessage($query, __FILE__, __LINE__, PEAR_LOG_ERR);
-            return $query;
+        $stmt = $db->prepare('DELETE FROM ' . $table . '_groups WHERE share_id = ?', null, MDB2_PREPARE_MANIP);
+        if (is_a($stmt, 'PEAR_Error')) {
+            Horde::logMessage($stmt, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return $stmt;
         }
-        $result = $query->execute(array($this->data['share_id']));
+        $result = $stmt->execute(array($this->data['share_id']));
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
         }
+        $stmt->free();
 
         if (!empty($this->data['perm']['groups'])) {
             $data = array();
-            foreach ($this->data['perm']['groups'] as $user => $perm) {
-                $data[] = array($this->data['share_id'], $user, $perm);
-            }
-            $sth = $db->prepare('INSERT INTO ' . $table . '_groups (share_id, group_uid, perm) VALUES (?, ?, ?)', null, MDB2_PREPARE_MANIP);
-            if (is_a($sth, 'PEAR_Error')) {
-                Horde::logMessage($sth, __FILE__, __LINE__, PEAR_LOG_ERR);
-                return $sth;
-            }
-            $result = $db->extended->executeMultiple($sth, $data);
-            if (is_a($result, 'PEAR_Error')) {
-                Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
-                return $result;
+            foreach ($this->data['perm']['groups'] as $group => $perm) {
+                $stmt = $db->prepare('INSERT INTO ' . $table . '_groups (share_id, group_uid, perm) VALUES (?, ?, ?)', null, MDB2_PREPARE_MANIP);
+                if (is_a($stmt, 'PEAR_Error')) {
+                    Horde::logMessage($stmt, __FILE__, __LINE__, PEAR_LOG_ERR);
+                    return $stmt;
+                }
+                $result = $stmt->execute(array($this->data['share_id'], $group, $perm));
+                if (is_a($result, 'PEAR_Error')) {
+                    Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+                    return $result;
+                }
+                $stmt->free();
             }
         }
 
