@@ -165,9 +165,9 @@
  *    testTearDown() to create a clean (empty) enviroment for the test user
  *    "syncmltest".  See the SyncML_Backend_Sql implementation for details.
  *
- * $Horde: framework/SyncML/SyncML/Backend.php,v 1.8.2.11 2008/04/08 23:53:08 jan Exp $
+ * $Horde: framework/SyncML/SyncML/Backend.php,v 1.8.2.21 2009/12/30 01:15:21 jan Exp $
  *
- * Copyright 2005-2007 Karsten Fourmont <karsten@horde.org>
+ * Copyright 2005-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -194,7 +194,7 @@ class SyncML_Backend {
      *
      * @var string
      */
-    var $_logtext;
+    var $_logtext = '';
 
     /**
      * The directory where debugging information is stored.
@@ -219,7 +219,7 @@ class SyncML_Backend {
      * @see SyncML_Backend()
      * @var integer
      */
-    var $_logLevel;
+    var $_logLevel = PEAR_LOG_INFO;
 
     /**
      * The charset used in the SyncML messages.
@@ -228,13 +228,6 @@ class SyncML_Backend {
      */
     var $_charset;
 
-    /**
-     * The current packet number, used for debug output.
-     *
-     * @var integer
-     */
-    var $_packetNum;
-
     /**
      * The current user.
      *
@@ -284,16 +277,11 @@ class SyncML_Backend {
     {
         if (!empty($params['debug_dir']) && is_dir($params['debug_dir'])) {
             $this->_debugDir = $params['debug_dir'];
-            $this->_logtext = '';
-        } else {
-            $this->_debugDir = null;
-            $this->_logtext = null;
         }
-
         $this->_debugFiles = !empty($params['debug_files']);
-        $this->_logLevel = isset($params['log_level'])
-            ? $params['log_level']
-            : PEAR_LOG_INFO;
+        if (isset($params['log_level'])) {
+            $this->_logLevel = $params['log_level'];
+        }
 
         $this->logMessage('Backend of class ' . get_class($this) . ' created',
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
@@ -401,6 +389,16 @@ class SyncML_Backend {
         return $this->_user;
     }
 
+    /**
+     * Is called after the SyncML_State object has been set up, either
+     * restored from the session, or freshly created.
+     *
+     * @param SyncML_State  The current state object.
+     */
+    function setupState(&$state)
+    {
+    }
+
     /**
      * Starts a PHP session.
      *
@@ -455,15 +453,16 @@ class SyncML_Backend {
         $database = $this->_normalize($databaseURI);
 
         switch($database) {
-        case 'tasks';
-        case 'calendar';
-        case 'notes';
-        case 'contacts';
+        case 'tasks':
+        case 'calendar':
+        case 'notes':
+        case 'contacts':
+        case 'configuration':
             return true;
 
         default:
-            $this->logMessage('Invalid database ' . $database
-                              . '. Try tasks, calendar, notes or contacts.',
+            $this->logMessage('Invalid database "' . $database
+                              . '". Try tasks, calendar, notes or contacts.',
                               __FILE__, __LINE__, PEAR_LOG_ERR);
             return false;
         }
@@ -491,7 +490,7 @@ class SyncML_Backend {
     function getServerChanges($databaseURI, $from_ts, $to_ts, &$adds, &$mods,
                               &$dels)
     {
-        die('Not implemented!');
+        die('getServerChanges() not implemented!');
     }
 
     /**
@@ -507,12 +506,14 @@ class SyncML_Backend {
      *                             this is the guid.
      * @param string $contentType  Content-Type: the MIME type in which the
      *                             function should return the data.
+     * @param array $fields        Hash of field names and SyncML_Property
+     *                             properties with the requested fields.
      *
      * @return mixed  A string with the data entry or a PEAR_Error object.
      */
-    function retrieveEntry($databaseURI, $suid, $contentType)
+    function retrieveEntry($databaseURI, $suid, $contentType, $fields)
     {
-        die('Not implemented!');
+        die('retrieveEntry() not implemented!');
     }
 
     /**
@@ -532,7 +533,7 @@ class SyncML_Backend {
      */
     function addEntry($databaseURI, $content, $contentType, $cuid)
     {
-        die('Not implemented!');
+        die('addEntry() not implemented!');
     }
 
     /**
@@ -552,7 +553,7 @@ class SyncML_Backend {
      */
     function replaceEntry($databaseURI, $content, $contentType, $cuid)
     {
-        die('Not implemented!');
+        die('replaceEntry() not implemented!');
     }
 
     /**
@@ -570,7 +571,7 @@ class SyncML_Backend {
      */
     function deleteEntry($databaseURI, $cuid)
     {
-        die('Not implemented!');
+        die('deleteEntry() not implemented!');
     }
 
     /**
@@ -582,8 +583,6 @@ class SyncML_Backend {
      * username that is used for the session, overriding any username
      * specified in <LocName>.
      *
-     * @abstract
-     *
      * @param string $username    Username as provided in the <SyncHdr>.
      *                            May be overwritten by $credData.
      * @param string $credData    Authentication data provided by <Cred><Data>
@@ -594,11 +593,87 @@ class SyncML_Backend {
      *                            in the <SyncHdr>. Typically
      *                            'syncml:auth-basic'.
      *
-     * @return boolean  True if authentication succeeded.
+     * @return boolean|string  The user name if authentication succeeded, false
+     *                         otherwise.
      */
     function checkAuthentication(&$username, $credData, $credFormat, $credType)
     {
-        die('Not implemented!');
+        if (empty($credData) || empty($credType)) {
+            return false;
+        }
+
+        switch ($credType) {
+        case 'syncml:auth-basic':
+            list($username, $pwd) = explode(':', base64_decode($credData), 2);
+            $this->logMessage('Checking authentication for user ' . $username,
+                              __FILE__, __LINE__, PEAR_LOG_DEBUG);
+            return $this->_checkAuthentication($username, $pwd);
+
+        case 'syncml:auth-md5':
+            /* syncml:auth-md5 only transfers hash values of passwords.
+             * Currently the syncml:auth-md5 hash scheme is not supported
+             * by the authentication backend. So we can't use Horde to do
+             * authentication. Instead here is a very crude direct manual hook:
+             * To allow authentication for a user 'dummy' with password 'sync',
+             * run
+             * php -r 'print base64_encode(pack("H*",md5("dummy:sync")));'
+             * from the command line. Then create an entry like
+             *  'dummy' => 'ZD1ZeisPeQs0qipHc9tEsw==' in the users array below,
+             * where the value is the command line output.
+             * This user/password combination is then accepted for md5-auth.
+             */
+            $users = array(
+                  // example for user dummy with pass pass:
+                  // 'dummy' => 'ZD1ZeisPeQs0qipHc9tEsw=='
+                          );
+            if (empty($users[$username])) {
+                return false;
+            }
+
+            // @todo: nonce may be specified by client. Use it then.
+            $nonce = '';
+            if (base64_encode(pack('H*', md5($users[$username] . ':' . $nonce))) === $credData) {
+                return $this->_setAuthenticated($username, $credData);
+            }
+            return false;
+
+        default:
+            $this->logMessage('Unsupported authentication type ' . $credType,
+                              __FILE__, __LINE__, PEAR_LOG_ERR);
+            return false;
+        }
+    }
+
+    /**
+     * Authenticates the user at the backend.
+     *
+     * @abstract
+     *
+     * @param string $username    A user name.
+     * @param string $password    A password.
+     *
+     * @return boolean|string  The user name if authentication succeeded, false
+     *                         otherwise.
+     */
+    function _checkAuthentication($username, $password)
+    {
+        die('_checkAuthentication() not implemented!');
+    }
+
+    /**
+     * Sets a user as being authenticated at the backend.
+     *
+     * @abstract
+     *
+     * @param string $username    A user name.
+     * @param string $credData    Authentication data provided by <Cred><Data>
+     *                            in the <SyncHdr>.
+     *
+     * @return string  The user name.
+     */
+    function setAuthenticated($username, $credData)
+    {
+        die('setAuthenticated() not implemented!');
     }
 
     /**
@@ -852,6 +927,7 @@ class SyncML_Backend {
                 fclose($f);
             }
         }
+        session_write_close();
     }
 
     /**
@@ -882,7 +958,17 @@ class SyncML_Backend {
      */
     function testSetup($user, $pwd)
     {
-        die('Not implemented!');
+        die('testSetup() not implemented!');
+    }
+
+    /**
+     * Prepares the test start.
+     *
+     * @param string $user This user accout has to be created in the backend.
+     */
+    function testStart($user)
+    {
+        die('testStart() not implemented!');
     }
 
     /**
@@ -894,7 +980,7 @@ class SyncML_Backend {
      */
     function testTearDown()
     {
-        die('Not implemented!');
+        die('testTearDown() not implemented!');
     }
 
     /**
