@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: framework/SyncML/SyncML/Sync.php,v 1.8.4.14 2008/04/09 17:13:21 jan Exp $
+ * $Horde: framework/SyncML/SyncML/Sync.php,v 1.8.4.25 2009/12/30 01:15:21 jan Exp $
  *
- * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2003-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -34,6 +34,11 @@ class SyncML_Sync {
      */
     var $_sourceLocURI;
 
+    /**
+     * The synchronization method, one of the ALERT_* constants.
+     *
+     * @var integer
+     */
     var $_syncType;
 
     /**
@@ -79,12 +84,24 @@ class SyncML_Sync {
     var $_serverAnchorNext;
 
     /**
-     * Counts for [server|client] adds/replaces/deletes and errors.
+     * Number of objects that have been sent to the server for adding.
      *
      * @var integer
      */
     var $_client_add_count = 0;
+
+    /**
+     * Number of objects that have been sent to the server for replacement.
+     *
+     * @var integer
+     */
     var $_client_replace_count = 0;
+
+    /**
+     * Number of objects that have been sent to the server for deletion.
+     *
+     * @var integer
+     */
     var $_client_delete_count = 0;
 
     /**
@@ -95,20 +112,72 @@ class SyncML_Sync {
      */
     var $_client_addreplaces = 0;
 
+    /**
+     * Number of objects that have been sent to the client for adding.
+     *
+     * @var integer
+     */
     var $_server_add_count = 0;
+
+    /**
+     * Number of objects that have been sent to the client for replacement.
+     *
+     * @var integer
+     */
     var $_server_replace_count = 0;
+
+    /**
+     * Number of objects that have been sent to the client for deletion.
+     *
+     * @var integer
+     */
     var $_server_delete_count = 0;
+
+    /**
+     * Number of failed actions, for logging purposes only.
+     *
+     * @var integer
+     */
     var $_errors = 0;
 
+    /**
+     * List of object UIDs (in the keys) that have been added on the server
+     * since the last synchronization and are supposed to be sent to the
+     * client.
+     *
+     * @var array
+     */
     var $_server_adds;
+
+    /**
+     * List of object UIDs (in the keys) that have been changed on the server
+     * since the last synchronization and are supposed to be sent to the
+     * client.
+     *
+     * @var array
+     */
     var $_server_replaces;
+
+    /**
+     * List of object UIDs (in the keys) that have been deleted on the server
+     * since the last synchronization and are supposed to be sent to the
+     * client.
+     *
+     * @var array
+     */
     var $_server_deletes;
 
     /**
-     * For clients handling tasks and events in one database (like P900), we
-     * need to seperately store the server tasks adds. So when we get a map
-     * command from the client, we know wheter to put this in tasks or
-     * calendar.
+     * List of task UIDs (in the keys) that have been added on the server
+     * since the last synchronization and are supposed to be sent to the
+     * client.
+     *
+     * This is only used for clients handling tasks and events in one
+     * database. We need to seperately store the server tasks adds, so when we
+     * get a Map command from the client, we know whether to put this in tasks
+     * or calendar.
+     *
+     * @var array
      */
     var $_server_task_adds;
 
@@ -126,7 +195,7 @@ class SyncML_Sync {
     /**
      * Here's where the actual processing of a client-sent Sync Item takes
      * place. Entries are added, deleted or replaced from the server database
-     * by using Horde API (Registry) calls.
+     * by using backend API calls.
      *
      * @todo maybe this should be moved to SyncItem
      *
@@ -150,7 +219,7 @@ class SyncML_Sync {
                  * conformant (and wrong). */
                 strlen($item->content) + 1 != $item->size) {
                 $item->responseCode = RESPONSE_SIZE_MISMATCH;
-                $GLOBALS['backend']->logMessage(
+                $backend->logMessage(
                     'Item size mismatch. Size reported as ' . $item->size
                     . ' but actual size is ' . strlen($item->content),
                     __FILE__, __LINE__, PEAR_LOG_ERR);
@@ -168,13 +237,13 @@ class SyncML_Sync {
 
         if (($item->contentType == 'text/calendar' ||
              $item->contentType == 'text/x-vcalendar') &&
-            stristr($GLOBALS['backend']->_normalize($database), 'calendar') !== false && 
+            $backend->_normalize($database) == 'calendar' && 
             $device->handleTasksInCalendar()) {
             $tasksincalendar = true;
             /* Check if the client sends us a vtodo in a calendar sync. */
             if (preg_match('/(\r\n|\r|\n)BEGIN[^:]*:VTODO/',
                            "\n" . $content)) {
-                $hordedatabase = $this->_taskdDbUriForCalendarDbUri($database);
+                $hordedatabase = $this->_taskToCalendar($backend->_normalize($database));
              }
         } else {
             $tasksincalendar = false;
@@ -195,7 +264,7 @@ class SyncML_Sync {
         $cuid = $item->cuid;
         $suid = false;
 
-        if ($item->elementType =='Add') {
+        if ($item->elementType == 'Add') {
             /* Handle client add requests.
              *
              * @todo: check if this $cuid is already present and then maybe do
@@ -212,14 +281,14 @@ class SyncML_Sync {
                 $item->responseCode = RESPONSE_NO_EXECUTED;
                 $backend->logMessage('Error in adding client entry: ' . $suid->message, __FILE__, __LINE__, PEAR_LOG_ERR);
             }
-        } elseif ($item->elementType =='Delete') {
+        } elseif ($item->elementType == 'Delete') {
             /* Handle client delete requests. */
             $ok = $backend->deleteEntry($database, $cuid);
             if (!$ok && $tasksincalendar) {
                 $backend->logMessage(
                     'Task ' . $cuid . ' deletion sent with calendar request',
                     __FILE__, __LINE__, PEAR_LOG_DEBUG);
-                $ok = $backend->deleteEntry($this->_taskdDbUriForCalendarDbUri($database), $cuid);
+                $ok = $backend->deleteEntry($this->_taskToCalendar($backend->_normalize($database)), $cuid);
             }
 
             if ($ok) {
@@ -274,6 +343,8 @@ class SyncML_Sync {
 
     /**
      * Creates a <Sync> output containing the server changes.
+     *
+     * @todo Check for Mem/FreeMem and Mem/FreeID when checking MaxObjSize
      */
     function createSyncOutput(&$output)
     {
@@ -297,15 +368,21 @@ class SyncML_Sync {
 
         /* $messageFull will be set to true to indicate that there's no room
          * for other data in this message. If it's false (empty) and there are
-         * pending Sync data, the final command will sent the pending data.
-         *
-         * @todo: This global data should be moved to a global object. */
+         * pending Sync data, the final command will sent the pending data. */
         $messageFull = false;
 
         $state = &$_SESSION['SyncML.state'];
         $device = $state->getDevice();
-        $contentType = $device->getPreferredContentTypeClient($this->_targetLocURI,
-                                                              $this->_sourceLocURI);
+        $contentType = $device->getPreferredContentTypeClient(
+            $this->_targetLocURI, $this->_sourceLocURI);
+        $contentTypeTasks = $device->getPreferredContentTypeClient(
+            'tasks', $this->_sourceLocURI);
+        if ($state->deviceInfo && $state->deviceInfo->CTCaps) {
+            $fields = array($contentType => isset($state->deviceInfo->CTCaps[$contentType]) ? $state->deviceInfo->CTCaps[$contentType] : null,
+                            $contentTypeTasks => isset($state->deviceInfo->CTCaps[$contentTypeTasks]) ? $state->deviceInfo->CTCaps[$contentTypeTasks] : null);
+        } else {
+            $fields = array($contentType => null, $contentTypeTasks => null);
+        }
 
         /* If server modifications are not retrieved yet (first Sync element),
          * do it now. */
@@ -316,27 +393,27 @@ class SyncML_Sync {
                 . ' to ' . date('Y-m-d H:i:s', $this->_serverAnchorNext),
                 __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-            $r = $this->_retrieveChanges($this->_targetLocURI,
-                                         $this->_server_adds,
-                                         $this->_server_replaces,
-                                         $this->_server_deletes);
-            if (is_a($r, 'PEAR_Error')) {
+            $result = $this->_retrieveChanges($this->_targetLocURI,
+                                              $this->_server_adds,
+                                              $this->_server_replaces,
+                                              $this->_server_deletes);
+            if (is_a($result, 'PEAR_Error')) {
                 return;
             }
 
             /* If tasks are handled inside calendar, do the same again for
              * tasks. Merge resulting arrays. */
-            if (stristr($GLOBALS['backend']->_normalize($this->_targetLocURI), 'calendar') !== false && 
+            if ($backend->_normalize($this->_targetLocURI) == 'calendar' && 
                 $device->handleTasksInCalendar()) {
                 $backend->logMessage('Handling tasks in calendar sync',
                                      __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
                 $this->_server_task_adds = $deletes2 = $replaces2 = array();
-                $r = $this->_retrieveChanges('tasks',
-                                             $this->_server_task_adds,
-                                             $replaces2,
-                                             $deletes2);
-                if (is_a($r, 'PEAR_Error')) {
+                $result = $this->_retrieveChanges('tasks',
+                                                  $this->_server_task_adds,
+                                                  $replaces2,
+                                                  $deletes2);
+                if (is_a($result, 'PEAR_Error')) {
                     return;
                 }
                 $this->_server_adds = array_merge($this->_server_adds,
@@ -345,10 +422,6 @@ class SyncML_Sync {
                                                       $replaces2);
                 $this->_server_deletes = array_merge($this->_server_deletes,
                                                      $deletes2);
-
-                $contentType2 = $device->getPreferredContentTypeClient(
-                    'tasks',
-                    $this->_sourceLocURI);
             }
 
             $numChanges = count($this->_server_adds)
@@ -380,17 +453,8 @@ class SyncML_Sync {
          * another message from the client. */
         $GLOBALS['message_expectresponse'] = true;
 
-        /* Copy the changes so we can modify the $this->_server_adds etc
-         * inside the foreach loop by deleteing completed elements. */
-        $adds     = $this->_server_adds;
-        $replaces = $this->_server_replaces;
-        $deletes  = $this->_server_deletes;
-
-        if (count($adds) >0) {
-            $this->_expectingMapData = true;
-        }
-
         /* Handle deletions. */
+        $deletes = $this->_server_deletes;
         foreach ($deletes as $suid => $cuid) {
             /* Check if we have space left in the message. */
             if ($state->maxMsgSize - $output->getOutputSize() < MSG_TRAILER_LEN) {
@@ -404,24 +468,26 @@ class SyncML_Sync {
                 $this->_syncsSent += 1;
                 return;
             }
-            unset($this->_server_deletes[$suid]);
             $backend->logMessage(
                 "Sending delete from server: client id $cuid, server id $suid",
                 __FILE__, __LINE__, PEAR_LOG_DEBUG);
             /* Create a Delete request for client. */
-            $output->outputSyncCommand('Delete', null, null, null, $cuid, null);
+            $cmdId = $output->outputSyncCommand('Delete', null, null, null, $cuid, null);
+            unset($this->_server_deletes[$suid]);
+            $state->serverChanges[$state->messageID][$this->_targetLocURI][$cmdId] = array($suid, $cuid);
             $this->_server_delete_count++;
         }
 
         /* Handle additions. */
+        $adds = $this->_server_adds;
         foreach ($adds as $suid => $cuid) {
             $backend->logMessage("Sending add from server: $suid",
                                  __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
             $syncDB = isset($this->_server_task_adds[$suid]) ? 'tasks' : $this->_targetLocURI;
-            $ct = isset($this->_server_task_adds[$suid]) ? $contentType2 : $contentType;
+            $ct = isset($this->_server_task_adds[$suid]) ? $contentTypeTasks : $contentType;
 
-            $c = $backend->retrieveEntry($syncDB, $suid, $ct);
+            $c = $backend->retrieveEntry($syncDB, $suid, $ct, $fields[$ct]);
             /* Item in history but not in database. Strange, but can
              * happen. */
             if (is_a($c, 'PEAR_Error')) {
@@ -431,7 +497,7 @@ class SyncML_Sync {
                     __FILE__, __LINE__, PEAR_LOG_ERR);
             } else {
                 list($clientContent, $clientContentType, $clientEncodingType) =
-                    $device->convertServer2Client($c, $contentType);
+                    $device->convertServer2Client($c, $contentType, $syncDB);
                 /* Check if we have space left in the message. */
                 if (($state->maxMsgSize - $output->getOutputSize() - strlen($clientContent)) < MSG_TRAILER_LEN) {
                     $backend->logMessage(
@@ -446,6 +512,7 @@ class SyncML_Sync {
                         /* @todo: implement partial sending instead of
                          * dropping item! */
                         unset($this->_server_adds[$suid]);
+                        continue;
                     }
                     $messageFull = true;
                     $output->outputSyncEnd();
@@ -455,62 +522,65 @@ class SyncML_Sync {
 
                 /* @todo: on SlowSync send Replace instead! */
                 // $output->outputSyncCommand($refts == 0 ? 'Replace' : 'Add',
-                $output->outputSyncCommand('Add', $clientContent,
-                                           $clientContentType,
-                                           $clientEncodingType,
-                                           null, $suid);
+                $cmdId = $output->outputSyncCommand('Add', $clientContent,
+                                                    $clientContentType,
+                                                    $clientEncodingType,
+                                                    null, $suid);
                 $this->_server_add_count++;
             }
             unset($this->_server_adds[$suid]);
+            $state->serverChanges[$state->messageID][$this->_targetLocURI][$cmdId] = array($suid, 0);
+        }
+
+        if ($this->_server_add_count) {
+            $this->_expectingMapData = true;
         }
 
         /* Handle Replaces. */
+        $replaces = $this->_server_replaces;
         foreach ($replaces as $suid => $cuid) {
-            if (!$cuid) {
-                // @todo: create an "add" here instead?
+            $syncDB = isset($replaces2[$suid]) ? 'tasks' : $this->_targetLocURI;
+            $ct = isset($replaces2[$suid]) ? $contentTypeTasks : $contentType;
+            $c = $backend->retrieveEntry($syncDB, $suid, $ct, $fields[$ct]);
+            if (is_a($c, 'PEAR_Error')) {
+                /* Item in history but not in database. */
                 unset($this->_server_replaces[$suid]);
                 continue;
             }
 
-            $syncDB = isset($replaces2[$suid]) ? 'tasks' : $this->_targetLocURI;
-            $ct = isset($replaces2[$suid]) ? $contentType2 : $contentType;
-
-            $c = $backend->retrieveEntry($syncDB, $suid, $ct);
-            /* Item in history but not in database. Strange, but can
-             * happen. */
-            if (!is_a($c, 'PEAR_Error')) {
+            $backend->logMessage(
+                "Sending replace from server: $suid",
+                __FILE__, __LINE__, PEAR_LOG_DEBUG);
+            list($clientContent, $clientContentType, $clientEncodingType) =
+                $device->convertServer2Client($c, $contentType, $syncDB);
+            /* Check if we have space left in the message. */
+            if (($state->maxMsgSize - $output->getOutputSize() - strlen($clientContent)) < MSG_TRAILER_LEN) {
                 $backend->logMessage(
-                    "Sending replace from server: $suid",
+                    'Maximum message size ' . $state->maxMsgSize
+                    . ' approached during replace; current size: '
+                    . $output->getOutputSize(),
                     __FILE__, __LINE__, PEAR_LOG_DEBUG);
-                list($clientContent, $clientContentType, $clientEncodingType) =
-                    $device->convertServer2Client($c, $contentType);
-                /* Check if we have space left in the message. */
-                if (($state->maxMsgSize - $output->getOutputSize() - strlen($clientContent)) < MSG_TRAILER_LEN) {
+                if (strlen($clientContent) + MSG_DEFAULT_LEN > $state->maxMsgSize) {
                     $backend->logMessage(
-                        'Maximum message size ' . $state->maxMsgSize
-                        . ' approached during replace; current size: '
-                        . $output->getOutputSize(),
-                        __FILE__, __LINE__, PEAR_LOG_DEBUG);
-                    if (strlen($clientContent) + MSG_DEFAULT_LEN > $state->maxMsgSize) {
-                        $backend->logMessage(
-                            'Data item won\'t fit into a single message. Partial sending not implemented yet. Item will not be sent!',
-                            __FILE__, __LINE__, PEAR_LOG_WARNING);
-                        /* @todo: implement partial sending instead of
-                         * dropping item! */
-                        unset($this->_server_replaces[$suid]);
-                    }
-                    $messageFull = true;
-                    $output->outputSyncEnd();
-                    $this->_syncsSent += 1;
-                    return;
+                        'Data item won\'t fit into a single message. Partial sending not implemented yet. Item will not be sent!',
+                        __FILE__, __LINE__, PEAR_LOG_WARNING);
+                    /* @todo: implement partial sending instead of
+                     * dropping item! */
+                    unset($this->_server_replaces[$suid]);
+                    continue;
                 }
-                $output->outputSyncCommand('Replace', $clientContent,
-                                           $clientContentType,
-                                           $clientEncodingType,
-                                           $cuid, null);
-                $this->_server_replace_count++;
+                $messageFull = true;
+                $output->outputSyncEnd();
+                $this->_syncsSent += 1;
+                return;
             }
+            $cmdId = $output->outputSyncCommand('Replace', $clientContent,
+                                                $clientContentType,
+                                                $clientEncodingType,
+                                                $cuid, null);
+            $this->_server_replace_count++;
             unset($this->_server_replaces[$suid]);
+            $state->serverChanges[$state->messageID][$this->_targetLocURI][$cmdId] = array($suid, $cuid);
         }
 
         /* Finished! Send closing </Sync>. */
@@ -518,39 +588,32 @@ class SyncML_Sync {
         $this->_syncsSent += 1;
     }
 
+    /**
+     * Retrieves and condenses the changes on the server side since the last
+     * synchronization.
+     *
+     * @param string $syncDB   The database being synchronized.
+     * @param array $adds      Will be set with the server-client-uid mappings
+     *                         of added objects.
+     * @param array $replaces  Will be set with the server-client-uid mappings
+     *                         of changed objects.
+     * @param array $deletes   Will be set with the server-client-uid mappings
+     *                         of deleted objects.
+     */
     function _retrieveChanges($syncDB, &$adds, &$replaces, &$deletes)
     {
-        global $backend;
-
         $adds = $replaces = $deletes = array();
-
-        $r = $backend->getServerChanges($syncDB,
-                                        $this->_serverAnchorLast,
-                                        $this->_serverAnchorNext,
-                                        $adds, $replaces, $deletes);
-
-        if (is_a($r, 'PEAR_Error')) {
-            $backend->logMessage($r, __FILE__, __LINE__, PEAR_LOG_ERR);
-            return $r;
+        if ($syncDB == 'configuration') {
+            return;
         }
-
-        // Reduce duplicate adds or replaces:
-        $copy = $adds;
-        foreach ($copy as $suid => $cuid) {
-            if (isset($deletes[$suid])) {
-                // Delete will be sent, no need for add.
-                unset($adds[$suid]);
-            }
+        $result = $GLOBALS['backend']->getServerChanges($syncDB,
+                                                        $this->_serverAnchorLast,
+                                                        $this->_serverAnchorNext,
+                                                        $adds, $replaces, $deletes);
+        if (is_a($result, 'PEAR_Error')) {
+            $GLOBALS['backend']->logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return $result;
         }
-        $copy = $replaces;
-        foreach ($copy as $suid => $cuid) {
-            if (isset($adds[$suid]) || isset($deletes[$suid])) {
-                // Delete or add will be sent, no need to modify.
-                unset($replaces[$suid]);
-            }
-        }
-
-        return true;
     }
 
     /**
@@ -563,7 +626,7 @@ class SyncML_Sync {
      *    or finish if ONE_WAY_FROM_CLIENT sync
      * c) Maps finished, completly done.
      */
-    function handleFinal(&$output)
+    function handleFinal(&$output, $debug = false)
     {
         switch ($this->_state) {
         case STATE_INIT:
@@ -590,7 +653,9 @@ class SyncML_Sync {
         case STATE_SYNC:
             /* Received all client Sync data, now we are allowed to send
              * server sync data. */
-            $this->createSyncOutput($output);
+            if (!$debug) {
+                $this->createSyncOutput($output);
+            }
 
             // FROM_CLIENT_SYNC doeesn't require a MAP package:
             if ($this->_syncType == ALERT_ONE_WAY_FROM_CLIENT ||
@@ -651,7 +716,7 @@ class SyncML_Sync {
                                               $this->_serverAnchorNext);
 
         $s = sprintf(
-            'Successful sync of database %s. Failures: %d; '
+            'Finished sync of database %s. Failures: %d; '
             . 'changes from client (Add, Replace, Delete, AddReplaces): %d, %d, %d, %d; '
             . 'changes from server (Add, Replace, Delete): %d, %d, %d',
             $this->_targetLocURI,
@@ -695,10 +760,10 @@ class SyncML_Sync {
     {
         $device = $_SESSION['SyncML.state']->getDevice();
 
-        if (stristr($GLOBALS['backend']->_normalize($databaseURI), 'calendar') !== false &&
+        if ($GLOBALS['backend']->_normalize($databaseURI) == 'calendar' &&
             $device->handleTasksInCalendar() &&
             isset($this->_server_task_adds[$suid])) {
-            $db = $this->_taskdDbUriForCalendarDbUri($databaseURI);
+            $db = $this->_taskToCalendar($GLOBALS['backend']->_normalize($databaseURI));
         } else {
             $db = $databaseURI;
         }
@@ -711,13 +776,55 @@ class SyncML_Sync {
 
     }
 
+    /**
+     * Returns the client ID of server change identified by the change type
+     * and server ID.
+     *
+     * @param string $change  The change type (add, replace, delete).
+     * @param string $id      The object's server UID.
+     *
+     * @return string  The matching client ID or null if none found.
+     */
+    function getServerChange($change, $id)
+    {
+        $property = '_server_' . $change . 's';
+        return isset($this->$property[$id]) ? $this->$property[$id] : null;
+    }
+
+    /**
+     * Sets the client ID of server change identified by the change type and
+     * server ID.
+     *
+     * @param string $change  The change type (add, replace, delete).
+     * @param string $sid     The object's server UID.
+     * @param string $cid     The object's client UID.
+     */
+    function setServerChange($change, $sid, $cid)
+    {
+        $property = '_server_' . $change . 's';
+        $this->$property[$sid] = $cid;
+    }
+
+    /**
+     * Unsets the server-client-map of server change identified by the change
+     * type and server ID.
+     *
+     * @param string $change  The change type (add, replace, delete).
+     * @param string $id      The object's server UID.
+     */
+    function unsetServerChange($change, $id)
+    {
+        $property = '_server_' . $change . 's';
+        unset($this->$property[$id]);
+    }
+
     /**
      * Converts a calendar databaseURI to a tasks databaseURI for devices with
      * handleTasksInCalendar.
      */
-    function _taskdDbUriForCalendarDbUri($databaseURI)
+    function _taskToCalendar($databaseURI)
     {
-        return preg_replace('/calendar/', 'tasks', $databaseURI);
+        return str_replace('calendar', 'tasks', $databaseURI);
     }
 
-}
\ No newline at end of file
+}
