@@ -11,9 +11,9 @@
  * The SyncML_XMLOutput class takes automatically care of creating a unique
  * CmdID for each command created.
  *
- * $Horde: framework/SyncML/SyncML/XMLOutput.php,v 1.14.2.3 2008/04/08 23:53:08 jan Exp $
+ * $Horde: framework/SyncML/SyncML/XMLOutput.php,v 1.14.2.8 2009/01/06 15:23:37 jan Exp $
  *
- * Copyright 2006-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2006-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -79,70 +79,69 @@ class SyncML_XMLOutput {
 
     }
 
-    /** creates a SyncHdr output.
-     * All required data is retrieved from state.
+    /**
+     * Creates a SyncHdr output.
+     *
+     * Required data is retrieved from state.
+     *
+     * @param string $respURI  The url of the server endpoint.
      */
-    function outputHeader()
+    function outputHeader($respURI)
     {
         $state = &$_SESSION['SyncML.state'];
-        $attrs = array();
         $this->_uriMeta = $state->uriMeta;
 
-        $this->_output->startElement($this->_uri, 'SyncHdr', $attrs);
+        $this->_output->startElement($this->_uri, 'SyncHdr');
 
-        $this->_output->startElement($this->_uri, 'VerDTD', $attrs);
+        $this->_output->startElement($this->_uri, 'VerDTD');
         $chars = $state->getVerDTD();
         $this->_output->characters($chars);
         $this->_output->endElement($this->_uri, 'VerDTD');
 
-        $this->_output->startElement($this->_uri, 'VerProto', $attrs);
+        $this->_output->startElement($this->_uri, 'VerProto');
         $chars = $state->getProtocolName();
         $this->_output->characters($chars);
         $this->_output->endElement($this->_uri, 'VerProto');
 
-        $this->_output->startElement($this->_uri, 'SessionID', $attrs);
+        $this->_output->startElement($this->_uri, 'SessionID');
         $this->_output->characters($state->sessionID);
         $this->_output->endElement($this->_uri, 'SessionID');
 
-        $this->_output->startElement($this->_uri, 'MsgID', $attrs);
+        $this->_output->startElement($this->_uri, 'MsgID');
         $this->_output->characters($state->messageID);
         $this->_output->endElement($this->_uri, 'MsgID');
 
-        $this->_output->startElement($this->_uri, 'Target', $attrs);
-        $this->_output->startElement($this->_uri, 'LocURI', $attrs);
+        $this->_output->startElement($this->_uri, 'Target');
+        $this->_output->startElement($this->_uri, 'LocURI');
         // Source URI sent from client is Target for the server
         $this->_output->characters($state->sourceURI);
         $this->_output->endElement($this->_uri, 'LocURI');
         if ($state->user) {
-            $this->_output->startElement($this->_uri, 'LocName', $attrs);
+            $this->_output->startElement($this->_uri, 'LocName');
             $this->_output->characters($state->user);
             $this->_output->endElement($this->_uri, 'LocName');
         }
         $this->_output->endElement($this->_uri, 'Target');
 
-        $this->_output->startElement($this->_uri, 'Source', $attrs);
-        $this->_output->startElement($this->_uri, 'LocURI', $attrs);
+        $this->_output->startElement($this->_uri, 'Source');
+        $this->_output->startElement($this->_uri, 'LocURI');
         // Target URI sent from client is Source for the server
         $this->_output->characters($state->targetURI);
         $this->_output->endElement($this->_uri, 'LocURI');
         $this->_output->endElement($this->_uri, 'Source');
 
-        /*
-        Do not send RespURI Element. It's optional and may be misleading:
-        $this->_targetURI is data from the client and not guaranteed to be
-        the correct URL for access. Let the client just stay with the same
-        URL it has used for the previous request(s).
-        */
-        //$this->_output->startElement($this->_uri, 'RespURI', $attrs);
-        //$this->_output->characters($state->targetURI);
-        //$this->_output->endElement($this->_uri, 'RespURI');
+        if ($respURI) {
+            $this->_output->startElement($this->_uri, 'RespURI');
+            $this->_output->characters($respURI);
+            $this->_output->endElement($this->_uri, 'RespURI');
+        }
 
         // @Todo: omit this in SyncML1.0?
-        $this->_output->startElement($this->_uri, 'Meta', $attrs);
+        $this->_output->startElement($this->_uri, 'Meta');
 
         // Dummy Max MsqSize, this is just put in to make the packet
         // work, it is not a real value.
-        $this->_output->startElement($this->_uriMeta, 'MaxMsgSize', $attrs);
+        $this->_output->startElement($this->_uriMeta, 'MaxMsgSize');
         $chars = SERVER_MAXMSGSIZE; // 1Meg
         $this->_output->characters($chars);
         $this->_output->endElement($this->_uriMeta, 'MaxMsgSize');
@@ -150,7 +149,7 @@ class SyncML_XMLOutput {
 
         // MaxObjSize, required by protocol for SyncML1.1 and higher.
         if ($state->version > 0) {
-            $this->_output->startElement($this->_uriMeta, 'MaxObjSize', $attrs);
+            $this->_output->startElement($this->_uriMeta, 'MaxObjSize');
             $this->_output->characters(SERVER_MAXOBJSIZE);
             $this->_output->endElement($this->_uriMeta, 'MaxObjSize');
         }
@@ -191,34 +190,33 @@ class SyncML_XMLOutput {
     {
         $state = &$_SESSION['SyncML.state'];
         $uriMeta = $state->uriMeta;
-        $attrs = array();
 
-        $this->_output->startElement($this->_uri, 'Status', $attrs);
+        $this->_output->startElement($this->_uri, 'Status');
         $this->_outputCmdID();
 
-        $this->_output->startElement($this->_uri, 'MsgRef', $attrs);
+        $this->_output->startElement($this->_uri, 'MsgRef');
         $chars = $state->messageID;
         $this->_output->characters($chars);
         $this->_output->endElement($this->_uri, 'MsgRef');
 
-        $this->_output->startElement($this->_uri, 'CmdRef', $attrs);
+        $this->_output->startElement($this->_uri, 'CmdRef');
         $chars = $cmdRef;
         $this->_output->characters($chars);
         $this->_output->endElement($this->_uri, 'CmdRef');
 
-        $this->_output->startElement($this->_uri, 'Cmd', $attrs);
+        $this->_output->startElement($this->_uri, 'Cmd');
         $chars = $cmd;
         $this->_output->characters($chars);
         $this->_output->endElement($this->_uri, 'Cmd');
 
         if (!empty($targetRef)) {
-            $this->_output->startElement($this->_uri, 'TargetRef', $attrs);
+            $this->_output->startElement($this->_uri, 'TargetRef');
             $this->_output->characters($targetRef);
             $this->_output->endElement($this->_uri, 'TargetRef');
         }
 
         if (!empty($sourceRef)) {
-            $this->_output->startElement($this->_uri, 'SourceRef', $attrs);
+            $this->_output->startElement($this->_uri, 'SourceRef');
             $this->_output->characters($sourceRef);
             $this->_output->endElement($this->_uri, 'SourceRef');
         }
@@ -226,19 +224,20 @@ class SyncML_XMLOutput {
         // If we are responding to the SyncHdr and we are not
         // authenticated then request basic authorization.
         if ($cmd == 'SyncHdr' && !$state->authenticated) {
-            // keep RESPONSE_CREDENTIALS_MISSING, otherwise set to RESPONSE_INVALID_CREDENTIALS
-            $data = ($data== RESPONSE_CREDENTIALS_MISSING)
+            // Keep RESPONSE_CREDENTIALS_MISSING, otherwise set to
+            // RESPONSE_INVALID_CREDENTIALS.
+            $data = $data == RESPONSE_CREDENTIALS_MISSING
                 ? RESPONSE_CREDENTIALS_MISSING
                 : RESPONSE_INVALID_CREDENTIALS;
 
-            $this->_output->startElement($this->_uri, 'Chal', $attrs);
-            $this->_output->startElement($this->_uri, 'Meta', $attrs);
+            $this->_output->startElement($this->_uri, 'Chal');
+            $this->_output->startElement($this->_uri, 'Meta');
 
-            $this->_output->startElement($uriMeta, 'Type', $attrs);
+            $this->_output->startElement($uriMeta, 'Type');
             $this->_output->characters('syncml:auth-basic');
             $this->_output->endElement($uriMeta, 'Type');
 
-            $this->_output->startElement($uriMeta, 'Format', $attrs);
+            $this->_output->startElement($uriMeta, 'Format');
             $this->_output->characters('b64');
             $this->_output->endElement($uriMeta, 'Format');
 
@@ -247,24 +246,24 @@ class SyncML_XMLOutput {
 
         }
 
-        $this->_output->startElement($this->_uri, 'Data', $attrs);
+        $this->_output->startElement($this->_uri, 'Data');
         $this->_output->characters($data);
         $this->_output->endElement($this->_uri, 'Data');
 
         if (!empty($syncAnchorNext) || !empty($syncAnchorNLast)) {
-            $this->_output->startElement($this->_uri, 'Item', $attrs);
-            $this->_output->startElement($this->_uri, 'Data', $attrs);
+            $this->_output->startElement($this->_uri, 'Item');
+            $this->_output->startElement($this->_uri, 'Data');
 
-            $this->_output->startElement($uriMeta, 'Anchor', $attrs);
+            $this->_output->startElement($uriMeta, 'Anchor');
 
             if (!empty($syncAnchorLast)) {
-              $this->_output->startElement($uriMeta, 'Last', $attrs);
+              $this->_output->startElement($uriMeta, 'Last');
               $this->_output->characters($syncAnchorLast);
               $this->_output->endElement($uriMeta, 'Last');
             }
 
             if (!empty($syncAnchorNext)) {
-              $this->_output->startElement($uriMeta, 'Next', $attrs);
+              $this->_output->startElement($uriMeta, 'Next');
               $this->_output->characters($syncAnchorNext);
               $this->_output->endElement($uriMeta, 'Next');
             }
@@ -284,23 +283,22 @@ class SyncML_XMLOutput {
         $state = &$_SESSION['SyncML.state'];
         $uriMeta = $state->uriMeta;
         $uriDevInf = $state->uriDevInf;
-        $attrs = array();
 
-        $this->_output->startElement($this->_uri, 'Results', $attrs);
+        $this->_output->startElement($this->_uri, 'Results');
         $this->_outputCmdID();
 
-        $this->_output->startElement($this->_uri, 'MsgRef', $attrs);
+        $this->_output->startElement($this->_uri, 'MsgRef');
         $chars = $state->messageID;
         $this->_output->characters($chars);
         $this->_output->endElement($this->_uri, 'MsgRef');
 
-        $this->_output->startElement($this->_uri, 'CmdRef', $attrs);
+        $this->_output->startElement($this->_uri, 'CmdRef');
         $chars = $cmdRef;
         $this->_output->characters($chars);
         $this->_output->endElement($this->_uri, 'CmdRef');
 
-        $this->_output->startElement($this->_uri, 'Meta', $attrs);
-        $this->_output->startElement($uriMeta, 'Type', $attrs);
+        $this->_output->startElement($this->_uri, 'Meta');
+        $this->_output->startElement($uriMeta, 'Type');
         if ($state->wbxml) {
             $this->_output->characters(MIME_SYNCML_DEVICE_INFO_WBXML);
         } else {
@@ -310,50 +308,50 @@ class SyncML_XMLOutput {
         $this->_output->endElement($uriMeta, 'Type');
         $this->_output->endElement($this->_uri, 'Meta');
 
-        $this->_output->startElement($this->_uri, 'Item', $attrs);
-        $this->_output->startElement($this->_uri, 'Source', $attrs);
-        $this->_output->startElement($this->_uri, 'LocURI', $attrs);
+        $this->_output->startElement($this->_uri, 'Item');
+        $this->_output->startElement($this->_uri, 'Source');
+        $this->_output->startElement($this->_uri, 'LocURI');
         $this->_output->characters($state->getDevInfURI());
         $this->_output->endElement($this->_uri, 'LocURI');
         $this->_output->endElement($this->_uri, 'Source');
 
-        $this->_output->startElement($this->_uri, 'Data', $attrs);
+        $this->_output->startElement($this->_uri, 'Data');
 
         /* DevInf data is stored in wbxml not as a seperate codepage but
          * rather as a complete wbxml stream as opaque data.  So we need a
          * new Handler. */
         $devinfoutput = $this->_output->createSubHandler();
 
-        $devinfoutput->startElement($uriDevInf , 'DevInf', $attrs);
-        $devinfoutput->startElement($uriDevInf , 'VerDTD', $attrs);
+        $devinfoutput->startElement($uriDevInf , 'DevInf');
+        $devinfoutput->startElement($uriDevInf , 'VerDTD');
         $devinfoutput->characters($state->getVerDTD());
-        $devinfoutput->endElement($uriDevInf , 'VerDTD', $attrs);
-        $devinfoutput->startElement($uriDevInf , 'Man', $attrs);
-        $devinfoutput->characters('The Horde Project (http://www.horde.org)');
-        $devinfoutput->endElement($uriDevInf , 'Man', $attrs);
-        $devinfoutput->startElement($uriDevInf , 'DevID', $attrs);
+        $devinfoutput->endElement($uriDevInf , 'VerDTD');
+        $devinfoutput->startElement($uriDevInf , 'Man');
+        $devinfoutput->characters('The Horde Project (http://www.horde.org/)');
+        $devinfoutput->endElement($uriDevInf , 'Man');
+        $devinfoutput->startElement($uriDevInf , 'DevID');
         $devinfoutput->characters(isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : 'localhost');
-        $devinfoutput->endElement($uriDevInf , 'DevID', $attrs);
-        $devinfoutput->startElement($uriDevInf , 'DevTyp', $attrs);
+        $devinfoutput->endElement($uriDevInf , 'DevID');
+        $devinfoutput->startElement($uriDevInf , 'DevTyp');
         $devinfoutput->characters('server');
-        $devinfoutput->endElement($uriDevInf , 'DevTyp', $attrs);
+        $devinfoutput->endElement($uriDevInf , 'DevTyp');
 
         if ($state->version > 0) {
-            $devinfoutput->startElement($uriDevInf , 'SupportLargeObjs', $attrs);
-            $devinfoutput->endElement($uriDevInf , 'SupportLargeObjs', $attrs);
+            $devinfoutput->startElement($uriDevInf , 'SupportLargeObjs');
+            $devinfoutput->endElement($uriDevInf , 'SupportLargeObjs');
 
-            $devinfoutput->startElement($uriDevInf , 'SupportNumberOfChanges', $attrs);
-            $devinfoutput->endElement($uriDevInf , 'SupportNumberOfChanges', $attrs);
+            $devinfoutput->startElement($uriDevInf , 'SupportNumberOfChanges');
+            $devinfoutput->endElement($uriDevInf , 'SupportNumberOfChanges');
         }
-        $this->_writeDataStore('notes', 'text/x-vnote', '1.1', $devinfoutput,
-                               array('text/plain' => '1.0'));
-        $this->_writeDataStore('contacts', 'text/x-vcard', '3.0', $devinfoutput,
-                               array('text/x-vcard' => '2.1'));
+        $this->_writeDataStore('notes', 'text/plain', '1.0', $devinfoutput);
+        $this->_writeDataStore('contacts', 'text/directory', '3.0',
+                               $devinfoutput, array('text/x-vcard' => '2.1'));
         $this->_writeDataStore('tasks', 'text/calendar', '2.0', $devinfoutput,
                                array('text/x-vcalendar' => '1.0'));
-        $this->_writeDataStore('calendar', 'text/calendar', '2.0', $devinfoutput,
+        $this->_writeDataStore('calendar', 'text/calendar', '2.0',
+                               $devinfoutput,
                                array('text/x-vcalendar' => '1.0'));
-        $devinfoutput->endElement($uriDevInf , 'DevInf', $attrs);
+        $devinfoutput->endElement($uriDevInf , 'DevInf');
 
         $this->_output->opaque($devinfoutput->getOutput());
         $this->_output->endElement($this->_uri, 'Data');
@@ -364,120 +362,118 @@ class SyncML_XMLOutput {
     /**
      * Writes DevInf data for one DataStore.
      *
-     * @param string $sourceref: data for SourceRef element.
-     * @param string $mimetype: data for &lt;(R|T)x-Pref&gt;&lt;CTType&gt;
-     * @param string $version: data for &lt;(R|T)x-Pref&gt;&lt;VerCT&gt;
-     * @param string &$output contenthandler that will received the output.
-     * @param array $additionaltypes: array of additional types for Tx and Rx;
-     *              format array('text/vcard' => '2.0')
+     * @param string $sourceref                 Data for <SourceRef> element.
+     * @param string $mimetype                  Data for <Rx-Pref><CTType> and
+     *                                          <Tx-Pref><CTType>.
+     * @param string $version                   Data for <Rx-Pref><VerCT> and
+     *                                          <Tx-Pref><VerCT>.
+     * @param XML_WBXML_ContentHandler $output  Content handler that will
+     *                                          received the output.
+     * @param array $additionaltypes            Array of additional types for
+     *                                          <Tx> and <Rx>; format
+     *                                          array('text/directory' => '3.0')
      */
-    function _writeDataStore($sourceref, $mimetype, $version,&$output,
-                             $additionaltypes = false)
+    function _writeDataStore($sourceref, $mimetype, $version, &$output,
+                             $additionaltypes = array())
     {
         $uriDevInf = $_SESSION['SyncML.state']->uriDevInf;
-        $attrs = array();
 
-        $output->startElement($uriDevInf , 'DataStore', $attrs);
-        $output->startElement($uriDevInf , 'SourceRef', $attrs);
+        $output->startElement($uriDevInf , 'DataStore');
+        $output->startElement($uriDevInf , 'SourceRef');
         $output->characters($sourceref);
-        $output->endElement($uriDevInf , 'SourceRef', $attrs);
+        $output->endElement($uriDevInf , 'SourceRef');
 
-        $output->startElement($uriDevInf , 'Rx-Pref', $attrs);
-        $output->startElement($uriDevInf , 'CTType', $attrs);
+        $output->startElement($uriDevInf , 'Rx-Pref');
+        $output->startElement($uriDevInf , 'CTType');
         $output->characters($mimetype);
-        $output->endElement($uriDevInf , 'CTType', $attrs);
-        $output->startElement($uriDevInf , 'VerCT', $attrs);
+        $output->endElement($uriDevInf , 'CTType');
+        $output->startElement($uriDevInf , 'VerCT');
         $output->characters($version);
-        $output->endElement($uriDevInf , 'VerCT', $attrs);
-        $output->endElement($uriDevInf , 'Rx-Pref', $attrs);
+        $output->endElement($uriDevInf , 'VerCT');
+        $output->endElement($uriDevInf , 'Rx-Pref');
 
-        if (is_array($additionaltypes)) {
-            foreach ($additionaltypes as $ct => $ctver){
-                $output->startElement($uriDevInf , 'Rx', $attrs);
-                $output->startElement($uriDevInf , 'CTType', $attrs);
-                $output->characters($ct);
-                $output->endElement($uriDevInf , 'CTType', $attrs);
-                $output->startElement($uriDevInf , 'VerCT', $attrs);
-                $output->characters($ctver);
-                $output->endElement($uriDevInf , 'VerCT', $attrs);
-                $output->endElement($uriDevInf , 'Rx', $attrs);
-            }
+        foreach ($additionaltypes as $ct => $ctver){
+            $output->startElement($uriDevInf , 'Rx');
+            $output->startElement($uriDevInf , 'CTType');
+            $output->characters($ct);
+            $output->endElement($uriDevInf , 'CTType');
+            $output->startElement($uriDevInf , 'VerCT');
+            $output->characters($ctver);
+            $output->endElement($uriDevInf , 'VerCT');
+            $output->endElement($uriDevInf , 'Rx');
         }
 
-        $output->startElement($uriDevInf , 'Tx-Pref', $attrs);
-        $output->startElement($uriDevInf , 'CTType', $attrs);
+        $output->startElement($uriDevInf , 'Tx-Pref');
+        $output->startElement($uriDevInf , 'CTType');
         $output->characters($mimetype);
-        $output->endElement($uriDevInf , 'CTType', $attrs);
-        $output->startElement($uriDevInf , 'VerCT', $attrs);
+        $output->endElement($uriDevInf , 'CTType');
+        $output->startElement($uriDevInf , 'VerCT');
         $output->characters($version);
-        $output->endElement($uriDevInf , 'VerCT', $attrs);
-        $output->endElement($uriDevInf , 'Tx-Pref', $attrs);
+        $output->endElement($uriDevInf , 'VerCT');
+        $output->endElement($uriDevInf , 'Tx-Pref');
 
-        if (is_array($additionaltypes)) {
-            foreach ($additionaltypes as $ct => $ctver){
-                $output->startElement($uriDevInf , 'Tx', $attrs);
-                $output->startElement($uriDevInf , 'CTType', $attrs);
-                $output->characters($ct);
-                $output->endElement($uriDevInf , 'CTType', $attrs);
-                $output->startElement($uriDevInf , 'VerCT', $attrs);
-                $output->characters($ctver);
-                $output->endElement($uriDevInf , 'VerCT', $attrs);
-                $output->endElement($uriDevInf , 'Tx', $attrs);
-            }
+        foreach ($additionaltypes as $ct => $ctver){
+            $output->startElement($uriDevInf , 'Tx');
+            $output->startElement($uriDevInf , 'CTType');
+            $output->characters($ct);
+            $output->endElement($uriDevInf , 'CTType');
+            $output->startElement($uriDevInf , 'VerCT');
+            $output->characters($ctver);
+            $output->endElement($uriDevInf , 'VerCT');
+            $output->endElement($uriDevInf , 'Tx');
         }
 
-        $output->startElement($uriDevInf , 'SyncCap', $attrs);
-        // We support all sync Types from 1-6:
-        // two way, slow, refresh|update from client|server
-        for ($i=1; $i<=6; ++$i) {
-            $output->startElement($uriDevInf , 'SyncType', $attrs);
+        $output->startElement($uriDevInf , 'SyncCap');
+        // We support all sync Types from 1-6: two way, slow, refresh|update
+        // from client|server
+        for ($i = 1; $i <= 6; ++$i) {
+            $output->startElement($uriDevInf , 'SyncType');
             $output->characters($i);
-            $output->endElement($uriDevInf , 'SyncType', $attrs);
+            $output->endElement($uriDevInf , 'SyncType');
         }
-        $output->endElement($uriDevInf , 'SyncCap', $attrs);
-        $output->endElement($uriDevInf , 'DataStore', $attrs);
+        $output->endElement($uriDevInf , 'SyncCap');
+        $output->endElement($uriDevInf , 'DataStore');
     }
 
     function outputAlert($alertCode, $clientDB = '', $serverDB = '', $lastAnchor = '', $nextAnchor = '')
     {
         $uriMeta = $_SESSION['SyncML.state']->uriMeta;
-        $attrs = array();
 
-        $this->_output->startElement($this->_uri, 'Alert', $attrs);
+        $this->_output->startElement($this->_uri, 'Alert');
         $this->_outputCmdID();
 
-        $this->_output->startElement($this->_uri, 'Data', $attrs);
+        $this->_output->startElement($this->_uri, 'Data');
         $chars = $alertCode;
         $this->_output->characters($chars);
         $this->_output->endElement($this->_uri, 'Data');
 
-        $this->_output->startElement($this->_uri, 'Item', $attrs);
+        $this->_output->startElement($this->_uri, 'Item');
 
         if (!empty($clientDB)) {
-            $this->_output->startElement($this->_uri, 'Target', $attrs);
-            $this->_output->startElement($this->_uri, 'LocURI', $attrs);
+            $this->_output->startElement($this->_uri, 'Target');
+            $this->_output->startElement($this->_uri, 'LocURI');
             $this->_output->characters($clientDB);
             $this->_output->endElement($this->_uri, 'LocURI');
             $this->_output->endElement($this->_uri, 'Target');
         }
 
         if (!empty($serverDB)) {
-            $this->_output->startElement($this->_uri, 'Source', $attrs);
-            $this->_output->startElement($this->_uri, 'LocURI', $attrs);
+            $this->_output->startElement($this->_uri, 'Source');
+            $this->_output->startElement($this->_uri, 'LocURI');
             $this->_output->characters($serverDB);
             $this->_output->endElement($this->_uri, 'LocURI');
             $this->_output->endElement($this->_uri, 'Source');
         }
 
-        $this->_output->startElement($this->_uri, 'Meta', $attrs);
+        $this->_output->startElement($this->_uri, 'Meta');
 
-        $this->_output->startElement($uriMeta, 'Anchor', $attrs);
+        $this->_output->startElement($uriMeta, 'Anchor');
 
-        $this->_output->startElement($uriMeta, 'Last', $attrs);
+        $this->_output->startElement($uriMeta, 'Last');
         $this->_output->characters($lastAnchor);
         $this->_output->endElement($uriMeta, 'Last');
 
-        $this->_output->startElement($uriMeta, 'Next', $attrs);
+        $this->_output->startElement($uriMeta, 'Next');
         $this->_output->characters($nextAnchor);
         $this->_output->endElement($uriMeta, 'Next');
 
@@ -486,7 +482,7 @@ class SyncML_XMLOutput {
 
         // MaxObjSize, required by protocol for SyncML1.1 and higher.
         if ($_SESSION['SyncML.state']->version > 0) {
-            $this->_output->startElement($uriMeta, 'MaxObjSize', $attrs);
+            $this->_output->startElement($uriMeta, 'MaxObjSize');
             $this->_output->characters(SERVER_MAXOBJSIZE);
             $this->_output->endElement($uriMeta, 'MaxObjSize');
         }
@@ -502,14 +498,12 @@ class SyncML_XMLOutput {
     {
         $state = &$_SESSION['SyncML.state'];
         $uriMeta = $state->uriMeta;
-        $attrs = array();
 
-        $this->_output->startElement($this->_uri, 'Get', $attrs);
+        $this->_output->startElement($this->_uri, 'Get');
         $this->_outputCmdID();
 
-        $this->_output->startElement($this->_uri, 'Meta', $attrs);
-        $this->_output->startElement($uriMeta, 'Type', $attrs);
-        $attrs = array();
+        $this->_output->startElement($this->_uri, 'Meta');
+        $this->_output->startElement($uriMeta, 'Type');
         if ($state->wbxml) {
             $chars = MIME_SYNCML_DEVICE_INFO_WBXML;
         } else {
@@ -519,9 +513,9 @@ class SyncML_XMLOutput {
         $this->_output->endElement($uriMeta, 'Type');
         $this->_output->endElement($this->_uri, 'Meta');
 
-        $this->_output->startElement($this->_uri, 'Item', $attrs);
-        $this->_output->startElement($this->_uri, 'Target', $attrs);
-        $this->_output->startElement($this->_uri, 'LocURI', $attrs);
+        $this->_output->startElement($this->_uri, 'Item');
+        $this->_output->startElement($this->_uri, 'Target');
+        $this->_output->startElement($this->_uri, 'LocURI');
         $this->_output->characters($state->getDevInfURI());
         $this->_output->endElement($this->_uri, 'LocURI');
         $this->_output->endElement($this->_uri, 'Target');
@@ -531,55 +525,60 @@ class SyncML_XMLOutput {
     }
 
     /**
-     * Output a single Sync command (Add, Delete, Replace).
+     * Creates a single Sync command
+     *
+     * @param string $command       The Sync command (Add, Delete, Replace).
+     * @param string $content       The actual object content.
+     * @param string $contentType   The content's MIME type.
+     * @param string $encodingType  The content encoding of the object.
+     * @param string $cuid          The client's object UID.
+     * @param string $suid          The server's object UID.
+     *
+     * @return integer  The CmdID used for this command.
      */
-    function outputSyncCommand($command,
-                           $content=null, $contentType = null,
-                           $encodingType = null,
-                           $cuid = null, $suid = null)
+    function outputSyncCommand($command, $content = null, $contentType = null,
+                               $encodingType = null, $cuid = null, $suid = null)
     {
         $uriMeta = $_SESSION['SyncML.state']->uriMeta;
-        $attrs = array();
 
-        $this->_output->startElement($this->_uri, $command, $attrs);
+        $this->_output->startElement($this->_uri, $command);
         $this->_outputCmdID();
 
         if (isset($contentType)) {
-            $this->_output->startElement($this->_uri, 'Meta', $attrs);
-            $this->_output->startElement($uriMeta, 'Type', $attrs);
+            $this->_output->startElement($this->_uri, 'Meta');
+            $this->_output->startElement($uriMeta, 'Type');
             $this->_output->characters($contentType);
             $this->_output->endElement($uriMeta, 'Type');
             $this->_output->endElement($this->_uri, 'Meta');
         }
 
-        if (isset($content)
-            || isset($cuid) || isset($suid)) {
-            $this->_output->startElement($this->_uri, 'Item', $attrs);
+        if (isset($content) || isset($cuid) || isset($suid)) {
+            $this->_output->startElement($this->_uri, 'Item');
             if ($suid != null) {
-                $this->_output->startElement($this->_uri, 'Source', $attrs);
-                $this->_output->startElement($this->_uri, 'LocURI', $attrs);
+                $this->_output->startElement($this->_uri, 'Source');
+                $this->_output->startElement($this->_uri, 'LocURI');
                 $this->_output->characters($suid);
                 $this->_output->endElement($this->_uri, 'LocURI');
                 $this->_output->endElement($this->_uri, 'Source');
             }
 
             if ($cuid != null) {
-                $this->_output->startElement($this->_uri, 'Target', $attrs);
-                $this->_output->startElement($this->_uri, 'LocURI', $attrs);
+                $this->_output->startElement($this->_uri, 'Target');
+                $this->_output->startElement($this->_uri, 'LocURI');
                 $this->_output->characters($cuid);
                 $this->_output->endElement($this->_uri, 'LocURI');
                 $this->_output->endElement($this->_uri, 'Target');
             }
 
             if (!empty($encodingType)) {
-                $this->_output->startElement($this->_uri, 'Meta', $attrs);
-                $this->_output->startElement($uriMeta, 'Format', $attrs);
+                $this->_output->startElement($this->_uri, 'Meta');
+                $this->_output->startElement($uriMeta, 'Format');
                 $this->_output->characters($encodingType);
                 $this->_output->endElement($uriMeta, 'Format');
                 $this->_output->endElement($this->_uri, 'Meta');
             }
             if (isset($content)) {
-                $this->_output->startElement($this->_uri, 'Data', $attrs);
+                $this->_output->startElement($this->_uri, 'Data');
                 if($this->isWBXML()) {
                     $this->_output->characters($content);
                 } else {
@@ -598,29 +597,29 @@ class SyncML_XMLOutput {
         }
 
         $this->_output->endElement($this->_uri, $command);
+
+        return $this->_msg_CmdID - 1;
     }
 
-    function outputSyncStart($clientLocURI,$serverLocURI, $numberOfChanges = null)
+    function outputSyncStart($clientLocURI, $serverLocURI, $numberOfChanges = null)
     {
-        $attrs = array();
-
-        $this->_output->startElement($this->_uri, 'Sync', $attrs);
+        $this->_output->startElement($this->_uri, 'Sync');
         $this->_outputCmdID();
 
-        $this->_output->startElement($this->_uri, 'Target', $attrs);
-        $this->_output->startElement($this->_uri, 'LocURI', $attrs);
+        $this->_output->startElement($this->_uri, 'Target');
+        $this->_output->startElement($this->_uri, 'LocURI');
         $this->_output->characters($clientLocURI);
         $this->_output->endElement($this->_uri, 'LocURI');
         $this->_output->endElement($this->_uri, 'Target');
 
-        $this->_output->startElement($this->_uri, 'Source', $attrs);
-        $this->_output->startElement($this->_uri, 'LocURI', $attrs);
+        $this->_output->startElement($this->_uri, 'Source');
+        $this->_output->startElement($this->_uri, 'LocURI');
         $this->_output->characters($serverLocURI);
         $this->_output->endElement($this->_uri, 'LocURI');
         $this->_output->endElement($this->_uri, 'Source');
 
         if (is_int($numberOfChanges)) {
-            $this->_output->startElement($this->_uri, 'NumberOfChanges', $attrs);
+            $this->_output->startElement($this->_uri, 'NumberOfChanges');
             $this->_output->characters($numberOfChanges);
             $this->_output->endElement($this->_uri, 'NumberOfChanges');
         }
@@ -635,14 +634,12 @@ class SyncML_XMLOutput {
 
     //  internal helper functions:
 
-    function _outputCmdID() {
-        $attrs = array();
-
-        $this->_output->startElement($this->_uri, 'CmdID', $attrs);
+    function _outputCmdID()
+    {
+        $this->_output->startElement($this->_uri, 'CmdID');
         $this->_output->characters($this->_msg_CmdID);
-        $this->_msg_CmdID += 1;
+        $this->_msg_CmdID++;
         $this->_output->endElement($this->_uri, 'CmdID');
-
     }
 
     /**
@@ -653,8 +650,7 @@ class SyncML_XMLOutput {
         if (empty($uri)) {
             $uri = $this->_uri;
         }
-        $attrs = array();
-        $this->_output->startElement($uri, $tag, $attrs);
+        $this->_output->startElement($uri, $tag);
         $this->_output->characters($str);
         $this->_output->endElement($uri, $tag);
     }
